@@ -1,5 +1,5 @@
 /* Functions common to all AIs.
-   Copyright (C) 1992-1997 Stanley T. Shebs.
+   Copyright (C) 1992-1997, 1999-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -10,37 +10,74 @@ any later version.  See the file COPYING.  */
 #include "kernel.h"
 /* (should do a better separation of kernel in set_side_ai) */
 #include "kpublic.h"
-extern void try_to_draw PARAMS ((Side *side, int flag, char *ainame));
-extern void give_up PARAMS ((Side *side, char *ainame));
-
-/* The number of AI types defined.  Increment for each new AI type
-   that is compiled in. */
-
-#define numaitypes 2
+#include "ai.h"
 
 /* Declarations of all AI types. */
 
 extern AI_ops mplayer_ops;
+extern AI_ops iplayer_ops;
+
+/* The array of all possible AI types. */
 
 AI_ops **all_ai_ops;
 
 void
-init_ai_types()
+init_ai_types(void)
 {
     /* Fill in the table of pointers to AI types. */
     all_ai_ops = (AI_ops **) xmalloc(numaitypes * sizeof(AI_ops *));
+    /* A dummy AI(?) - all methods are NULL */
     all_ai_ops[0] = (AI_ops *) xmalloc(sizeof(AI_ops));
+    /* The generic AI "mplayer". */
     all_ai_ops[1] = &mplayer_ops;
+    /* The independent AI "iplayer". */
+    all_ai_ops[2] = &iplayer_ops;
 }
 
+/* Given an AI type name, find its numerical index. */
+
+int
+find_ai_type(char *typename)
+{
+    int i;
+
+    for (i = 1; i < numaitypes; ++i) {
+	if (strcmp((all_ai_ops[i])->name, typename) == 0)
+	  return i;
+    }
+    return 0;
+}
+
+/* Given an AI type name, return the next name in the list of types,
+   or NULL if the name is the last in the list. */
+
+char *
+next_ai_type_name(char *typename)
+{
+    int i;
+    char *name;
+
+    if (typename == NULL)
+      return all_ai_ops[1]->name;
+    for (i = 1; i < numaitypes - 1; ++i) {
+	name = (all_ai_ops[i])->name;
+	if (strcmp(name, typename) == 0)
+	  return (all_ai_ops[i+1])->name;
+    }
+    return NULL;
+}
+
+/* Given a side that wants to have an AI running its units, find and
+   record the actual AI type that will be used, and run its
+   initialization method. */
+
 void
-init_ai(side)
-Side *side;
+init_ai(Side *side)
 {
     int i;
     char *typename;
-    int (*test) PARAMS ((void));
-    void (*fn) PARAMS ((Side *side));
+    int (*test)(void);
+    void (*fn)(Side *side);
 
     if (side_wants_ai(side)) {
 	/* (should merge with same bit in set_side_ai) */
@@ -48,18 +85,23 @@ Side *side;
 	    /* (should use the "best" default for this game and side) */
 	    side->player->aitypename = "mplayer";
 	}
+	/* If remote, do nothing beyond recording the name of the AI. */
+	if (numremotes > 0 && side->player->rid != my_rid)
+	  return;
+	/* Scan through the possible AI types. */
 	for (i = 0; i < numaitypes; ++i) {
 	    typename = (all_ai_ops[i])->name;
 	    if (typename != NULL
 		&& strcmp(typename, side->player->aitypename) == 0) {
 		test = (all_ai_ops[i])->to_test_compat;
 		if (test == NULL || (*test)()) {
+		    /* Record the index of the AI type. */
 		    side->aitype = i;
 		    fn = (all_ai_ops[i])->to_init;
 		    if (fn)
 		      (*fn)(side);
 		} else {
-		    /* desired aitype not avail - should complain */
+		    /* desired aitype incompatible - should complain */
 		}
 	    }
 	}
@@ -74,11 +116,10 @@ Side *side;
    being turned off. */
 
 void
-set_side_ai(side, typename)
-Side *side;
-char *typename;
+set_side_ai(Side *side, char *typename)
 {
     Unit *unit;
+    extern int num_local_ais;
 
     if (!empty_string(typename) && strcmp(typename, "ai") == 0) {
 	/* (should use the "best" default for this game and side) */
@@ -108,14 +149,25 @@ char *typename;
 	side->ai = NULL;
     } else {
 	init_ai(side);
+	/* Turn on automation for all units when switching on the
+	   side's AI. Note that the reverse is NOT true. Switching
+	   from AI to human control will therefore leave all units on
+	   automatic. */
+	for_all_side_units(side, unit) {
+	    unit->autoplan = TRUE;
+	    unit->autoresearch = TRUE;
+	    unit->autobuild = TRUE;
+	}
     }
+    num_local_ais = -1;
 }
 
+/* The following functions dispatch on AI type. */
+
 void
-ai_init_turn(side)
-Side *side;
+ai_init_turn(Side *side)
 {
-    void (*fn) PARAMS ((Side *side));
+    void (*fn)(Side *side);
 
     fn = (all_ai_ops[side->aitype])->to_init_turn;
     if (fn)
@@ -127,7 +179,7 @@ ai_decide_plan(side, unit)
 Side *side;
 Unit *unit;
 {
-    void (*fn) PARAMS ((Side *side, Unit *unit));
+    void (*fn)(Side *side, Unit *unit);
 
     fn = ((all_ai_ops[side->aitype])->to_decide_plan);
     if (fn)
@@ -142,7 +194,7 @@ Side *side;
 Unit *unit;
 int rslt;
 {
-    void (*fn) PARAMS ((Side *side, Unit *unit, int rslt));
+    void (*fn)(Side *side, Unit *unit, int rslt);
 
     fn = ((all_ai_ops[side->aitype])->to_react_to_action_result);
     if (fn)
@@ -158,7 +210,7 @@ Unit *unit;
 Task *task;
 TaskOutcome rslt;
 {
-    void (*fn) PARAMS ((Side *side, Unit *unit, Task *task, TaskOutcome rslt));
+    void (*fn)(Side *side, Unit *unit, Task *task, TaskOutcome rslt);
 
     fn = ((all_ai_ops[side->aitype])->to_react_to_task_result);
     if (fn)
@@ -169,7 +221,7 @@ void
 ai_react_to_new_side(side, side2)
 Side *side, *side2;
 {
-    void (*fn) PARAMS ((Side *side, Side *side2));
+    void (*fn)(Side *side, Side *side2);
 
     fn = ((all_ai_ops[side->aitype])->to_react_to_new_side);
     if (fn)
@@ -181,7 +233,7 @@ ai_adjust_plan(side, unit)
 Side *side;
 Unit *unit;
 {
-    int (*fn) PARAMS ((Side *side, Unit *unit));
+    int (*fn)(Side *side, Unit *unit);
 
     fn = ((all_ai_ops[side->aitype])->to_adjust_plan);
     if (fn)
@@ -194,7 +246,7 @@ void
 ai_finish_movement(side)
 Side *side;
 {
-    void (*fn) PARAMS ((Side *side));
+    void (*fn)(Side *side);
 
     fn = ((all_ai_ops[side->aitype])->to_finish_movement);
     if (fn)
@@ -208,7 +260,7 @@ ai_receive_message(side, sender, str)
 Side *side, *sender;
 char *str;
 {
-    void (*fn) PARAMS ((Side *side, Side *sender, char *str));
+    void (*fn)(Side *side, Side *sender, char *str);
 
     fn = ((all_ai_ops[side->aitype])->to_receive_message);
     if (fn)
@@ -219,7 +271,7 @@ void
 ai_save_state(side)
 Side *side;
 {
-    Obj *(*fn) PARAMS ((Side *side)), *state = lispnil;
+    Obj *(*fn)(Side *side), *state = lispnil;
 
     fn = ((all_ai_ops[side->aitype])->to_save_state);
     if (fn)
@@ -227,15 +279,20 @@ Side *side;
     /* Don't bother if there's no AI state to mess with. */
     if (side->aidata == lispnil && state == lispnil)
       return;
-    side->aidata = replace_at_key(side->aidata, side->player->aitypename, state);
+    side->aidata = replace_at_key(side->aidata, side->player->aitypename,
+				  state);
 }
 
+/* Return a numerical value indicate the AI's use of the given location.
+   For instance, each number could represent a region or theater of
+   operations. */
+
 int
 ai_region_at(side, x, y)
 Side *side;
 int x, y;
 {
-    int (*fn) PARAMS ((Side *side, int x, int y));
+    int (*fn)(Side *side, int x, int y);
 
     fn = ((all_ai_ops[side->aitype])->region_at);
     if (fn)
@@ -244,12 +301,15 @@ int x, y;
       return 0;
 }
 
+/* Provide textual information about the AI's usage or opinion of the
+   given location. */
+
 char *
 ai_at_desig(side, x, y)
 Side *side;
 int x, y;
 {
-    char *(*fn) PARAMS ((Side *side, int x, int y));
+    char *(*fn)(Side *side, int x, int y);
 
     fn = ((all_ai_ops[side->aitype])->at_desig);
     if (fn)
@@ -258,24 +318,22 @@ int x, y;
       return NULL;
 }
 
+/* Set willingness to declare a draw. */
+
 void
-try_to_draw(side, flag, ainame)
-Side *side;
-int flag;
-char *ainame;
+try_to_draw(Side *side, int flag, char *ainame)
 {
     /* If no change, nothing to say or do. */
     if (flag == side->willingtodraw)
       return;
-    /* If there is a human player, allow the human to make the final
-       decision. */
-    /* (should have a way to delegate this also, at least for testing purposes) */
-    if (side_has_display(side)) {
+    /* If there is a human player and the decision has not been
+       delegated, allow the human to make the final decision. */
+    if (side_has_display(side) && !side->ai_may_resign) {
 	if (flag)
-	  notify(side, "Your AI %s recommends being willing to draw",
+	  notify(side, "Your AI %s recommends being willing to draw.",
 		 ainame);
 	else
-	  notify(side, "Your AI %s recommends not being willing to draw",
+	  notify(side, "Your AI %s recommends not being willing to draw.",
 		 ainame);
 	return;
     }
@@ -285,31 +343,32 @@ char *ainame;
 /* If an AI resigns, it tries to help its friends. */
 
 void
-give_up(side, ainame)
-Side *side;
-char *ainame;
+give_up(Side *side, char *ainame)
 {
     Side *side1;
 
-    /* If there is a human player, allow the human to make the final
-       decision. */
-    /* (should have a way to delegate this also, at least for testing purposes) */
-    if (side_has_display(side)) {
+    /* The independents cannot resign unless played by a human. */
+    if (side == indepside && !side_has_display(side))
+      return;
+    /* Can't resign if we've been forbidden to do so. */
+    if (side_has_display(side) && !side->ai_may_resign) {
+	/* But speak up about it. */
 	notify(side, "Your AI %s recommends resignation", ainame);
 	return;
     }
-    /* (should have to give units to controlling side if there is one) */
-    /* (should controlling side be able to disallow resignation?) */
     /* Try to give away all of our units to an ally. */
     for_all_sides(side1) {
 	if (side != side1 && allied_side(side, side1) && side1->ingame) {
+	    if (side_has_display(side))
+	      notify(side, "Your AI resigns, giving its assets to %s!",
+		     short_side_title(side1));
 	    net_resign_game(side, side1);
 	    return;
 	}
     }
-    /* (should give to any positively-regarded side?) */
-    /* No allies left in game, let everything become independent. */
-    /* (should disband all units that can be disbanded?) */
+    /* No allies left in game, leave everything to its fate. */
+    if (side_has_display(side))
+      notify(side, "Your AI resigns!");
     net_resign_game(side, NULL);
 }
 
@@ -331,10 +390,7 @@ GoalDefn goaldefns[] = {
 /* General handling of goals. */
 
 Goal *
-create_goal(type, side, tf)
-GoalType type;
-Side *side;
-int tf;
+create_goal(GoalType type, Side *side, int tf)
 {
     Goal *goal = (Goal *) xmalloc(sizeof(Goal));
 
@@ -345,8 +401,7 @@ int tf;
 }
 
 int
-lookup_goal_type(name)
-char *name;
+lookup_goal_type(char *name)
 {
     int i;
 
@@ -357,15 +412,13 @@ char *name;
 }
 
 int
-cell_unknown(x, y)
-int x, y;
+cell_unknown(int x, int y)
 {
-    return (!all_see_all && !tmpside->see_all && terrain_view(tmpside, x, y) == UNSEEN);
+    return !(tmpside->see_all || terrain_view(tmpside, x, y) != UNSEEN);
 }
 
 int
-enemies_present(x, y)
-int x, y;
+enemies_present(int x, int y)
 {
     if (units_visible(tmpside, x, y)) {
     	return (unit_at(x, y) != NULL && !trusted_side(tmpside, unit_at(x, y)->side));
@@ -379,11 +432,9 @@ int x, y;
 /* Test a goal to see if it is true for side, as specified. */
 
 int
-goal_truth(side, goal)
-Side *side;
-Goal *goal;
+goal_truth(Side *side, Goal *goal)
 {
-    int x, y;
+    int x, y, m, n;
     Side *side2 = NULL;
 
     if (goal == NULL) return 0;
@@ -422,12 +473,21 @@ Goal *goal;
 	}
       case GOAL_CELL_OCCUPIED:
 	return 0;
+      case GOAL_UNIT_OCCUPIED:
+	return 0;
+      case GOAL_COLONIZE:
+	return 0;
       case GOAL_HAS_UNIT_TYPE:
 	return 0;
       case GOAL_HAS_UNIT_TYPE_NEAR:
 	return 0;
       case GOAL_HAS_MATERIAL_TYPE:
-	return 0;
+	m = goal->args[0];
+	n = goal->args[1];
+	if (side->treasury[m] >= n)
+	  return 100;
+	else
+	  return -100;
       default:
 	case_panic("goal type", goal->type);
 	return 0;
@@ -440,8 +500,7 @@ Goal *goal;
 char *goalbuf = NULL;
 
 char *
-goal_desig(goal)
-Goal *goal;
+goal_desig(Goal *goal)
 {
     int numargs, i, arg;
     char *argtypes;
@@ -542,21 +601,48 @@ ai_init_shared()
 	set_u_is_base_builder(u1, FALSE);
 	set_u_can_make(u1, FALSE);
 	set_u_can_capture(u1, FALSE);
+	set_u_is_ground_mobile(u1, FALSE);
+	set_u_is_naval_mobile(u1, FALSE);
+	set_u_is_air_mobile(u1, FALSE);
+	set_u_is_colonizer(u1, FALSE);
+	set_u_is_facility(u1, FALSE);
+
+	/* DON'T set u_advanced to FALSE. It is already set by the
+           game file. */
+
 	numbuilders = 0;
 /*	ave_build_time[u1] = 0;  */
 	for_all_unit_types(u2) {
 	    if (u_is_base(u2) &&
 		could_create(u1, u2) &&
 		1 /* can be made quickly? */) {
-		set_u_is_base_builder(u1, u2);
+
+// BUG FIX
+		set_u_is_base_builder(u1, TRUE);
+
+//		set_u_is_base_builder(u1, u2);
+
 	    }
 	    if (u_speed(u1) > 0 && could_carry(u1, u2)) {
 		set_u_is_transport(u1, TRUE);
 	    }
+
+// BUG FIX
+
+#if 1
+	    if (could_create(u1,u2)) {
+		set_u_can_make(u1, TRUE);
+#else
+
 	    if (could_create(u2, u1)) {
 		numbuilders++;
 /*		ave_build_time[u1] += uu_make(u2,u1);  */
 		set_u_can_make(u2, TRUE);
+
+#endif
+		/* A colonizer is a mobile builder of advanced units. */
+		if (u_advanced(u2) && mobile(u1))
+		  set_u_is_colonizer(u1, TRUE);
 	    }
 	    if (uu_capture(u1, u2) > 0 || uu_indep_capture(u1, u2) > 0) {
 		set_u_can_capture(u1, TRUE);
@@ -582,6 +668,49 @@ ai_init_shared()
     }
     for_all_unit_types(u) {
 	set_u_bw(u, basic_worth(u));
+	set_u_offensive_worth(u, offensive_worth(u));
+	set_u_defensive_worth(u, defensive_worth(u));
+	set_u_explorer_worth(u, exploring_worth(u));
+	set_u_colonizer_worth(u, colonizing_worth(u));
+	set_u_facility_worth(u, facility_worth(u));
+	set_u_random_worth(u, random_worth(u));
+	set_u_siege_worth(u, siege_worth(u));
+
+	if (mobile(u)) {
+	    int maxacp = u_acp(u);
+		
+	    if (u_acp_turn_max(u) >= 0)
+	      maxacp = min(maxacp, u_acp_turn_max(u));
+	    maxacp = (u_acp_max(u) < 0 ? maxacp : u_acp_max(u));
+	    /* Assume this is an air unit by default. */
+	    set_u_is_air_mobile(u, TRUE);
+	    for_all_terrain_types(t) {
+	        /* Check that our unit can have enough mps to enter the
+		   terrain. */
+	        if (ut_mp_to_enter(u, t) * u_acp_to_move(u) <= maxacp	
+		    /* And that there is room for our unit in the terrain. */
+		    && ut_size(u, t) <= t_capacity(t)
+		    /* And that it does not vanish in the terrain. */	
+		    && ut_vanishes_on(u, t) != TRUE) {
+		    /* It is naval mobile if it can enter at least one
+		       liquid terrain. */
+		    if (t_liquid(t))
+		      set_u_is_naval_mobile(u, TRUE);
+		    /* It is ground mobile if it can enter at least one
+		       non-liquid terrain. */
+		    if (!t_liquid(t))
+		      set_u_is_ground_mobile(u, TRUE);
+		    /* Air units should have zero size in all terrains. */
+		    if (ut_size(u, t) > 0)
+		      set_u_is_air_mobile(u, FALSE);
+		} else {
+		    /* Air units should be able to enter all terrains. */
+		    set_u_is_air_mobile(u, FALSE);
+	        }
+	    }
+	/* Kind of crude, but OK for now. */
+	} else
+	  set_u_is_facility(u, TRUE);
     }
     for_all_unit_types(u) {
 	for_all_unit_types(u2) {
@@ -598,8 +727,8 @@ ai_init_shared()
       display_assessment();
 }
 
-int basic_transport_worth PARAMS ((int u1, int u2));
-void set_uu_btw PARAMS ((int u1, int u2, int v));
+int basic_transport_worth(int u1, int u2);
+void set_uu_btw(int u1, int u2, int v);
 
 #define DICE(N,NUMDICE,SPOTS,OFFSET)  \
   (((N) >> 14 == 0 || (N) >> 14 == 3) ?  \
@@ -643,8 +772,131 @@ int u;
     return worth;
 }
 
-/* A basic estimate of the payoff of one unit type attacking another type
-   directly.  This is "context-free", does not account for overall goals etc. */
+int
+offensive_worth(int u)
+{
+    int offvalue = 0;
+    int defenders = 0;
+    int worth = 0;
+    int u2;
+
+    for_all_unit_types(u2) {
+	if (uu_acp_to_attack(u, u2) > 0) {
+	    ++defenders;
+	    offvalue += uu_hit(u, u2) * uu_damage(u, u2);
+	}
+	if (u_acp_to_fire(u) > 0) {
+	    ++defenders;
+	    offvalue += (uu_fire_hit(u, u2) != -1 ? uu_fire_hit(u, u2) : uu_hit(u, u2))
+	      * (uu_fire_damage(u, u2) != -1 ? uu_fire_damage(u, u2) : uu_damage(u, u2));
+	}
+    }
+    if (defenders > 0)
+      offvalue /= defenders;
+    worth =  u_acp(u) * offvalue * u_range(u) * u_speed(u) / 100;
+    DMprintf("unit type %s offensive worth %d \n ", u_type_name(u), worth);
+    return worth;
+}
+
+int
+defensive_worth(int u)
+{
+    int defvalue = 0;
+    int attackers = 0;
+    int worth = 0;
+    int u2;
+
+    for_all_unit_types(u2) {
+	if (uu_acp_to_attack(u2, u) > 0) {
+	    ++attackers;
+	    defvalue += 100 * u_hp_max(u) - uu_hit(u2, u) * uu_damage(u2, u);
+	}
+	if (u_acp_to_fire(u2) > 0) {
+	    ++attackers;   
+	    defvalue += 100 * u_hp_max(u) - (uu_fire_hit(u2, u) != -1 ? uu_fire_hit(u2, u) : uu_hit(u2, u)) 
+	      * (uu_fire_damage(u2, u) != -1 ? uu_fire_damage(u2, u) : uu_damage(u2, u));
+	}
+    }
+    if (attackers > 0)
+      defvalue /= attackers;
+    worth = u_acp(u) * defvalue / 10;
+    DMprintf("unit type %s defensive worth %d \n ", u_type_name(u), worth);
+    return worth;
+}
+
+int
+exploring_worth(int u)
+{
+    int worth = u_acp(u) * u_speed(u) / 100;
+
+    DMprintf("unit type %s exploring worth %d \n ", u_type_name(u), worth);
+    return worth;
+}
+
+int
+colonizing_worth(int u)
+{
+    int colvalue = 0;
+    int worth = 0;
+    int u2;
+ 
+    /* A colonizer is a mobile builder of advanced units. */
+    if (!mobile(u))
+      return 0;
+    for_all_unit_types(u2) {
+	if (could_create(u, u2)
+	    && (u_advanced(u2) || could_create(u2, u))) {
+	    /* It is more valuable if it can build bigger cities. */
+	    colvalue += u_reach(u2) + 1;
+	}
+    }    	
+    worth = u_acp(u) * u_speed(u) * colvalue / 10;
+    DMprintf("unit type %s colonizing worth %d \n ", u_type_name(u), worth);
+    return worth;
+}
+
+int
+facility_worth(int u)
+{
+    int worth = 0;
+
+    /* Kind of crude, but OK for now. */
+    if (!mobile(u))
+      worth = 1;
+
+    DMprintf("unit type %s facility worth %d \n ", u_type_name(u), worth);
+    return worth;
+}
+
+int
+siege_worth(int u)
+{
+    int siegevalue = 0;
+    int worth = 0;
+    int u2;
+ 
+    for_all_unit_types(u2) {
+	siegevalue += uu_capture(u, u2);
+    }           
+    siegevalue = siegevalue / numutypes;
+ 
+    worth = u_acp(u) * u_range(u) * siegevalue; 
+
+    DMprintf("unit type %s siege worth %d \n ", u_type_name(u), worth);
+    return worth;
+}
+
+/* Assign the same worth to all unit types. */
+
+int
+random_worth(int u)
+{
+    return 1;
+}
+
+/* A basic estimate of the payoff of one unit type attacking another
+   type directly.  This is "context-free", does not account for
+   overall goals etc. */
 
 /* (should account for number of attacks possible in one turn) */
 
@@ -814,6 +1066,21 @@ void set_u_can_make(u, n) int u, n; {  utypes[u].can_make = n;  }
 void set_u_can_capture(u, n) int u, n; {  utypes[u].can_capture = n;  }
 void set_u_bw(u, n) int u, n; {  utypes[u].bw = n;  }
 
+void set_u_offensive_worth(u, n) int u, n; {  utypes[u].ow = n;  }
+void set_u_defensive_worth(u, n) int u, n; {  utypes[u].dw = n;  }
+void set_u_explorer_worth(u, n) int u, n; {  utypes[u].ew = n;  }
+void set_u_colonizer_worth(u, n) int u, n; {  utypes[u].cw = n;  }
+void set_u_facility_worth(u, n) int u, n; {  utypes[u].fw = n;  }
+void set_u_random_worth(u, n) int u, n; {  utypes[u].rw = n;  }
+void set_u_siege_worth(u, n) int u, n; {  utypes[u].sw = n;  }
+
+void set_u_is_ground_mobile(u, n) int u, n; {  utypes[u].is_ground_mobile = n;  }
+void set_u_is_naval_mobile(u, n) int u, n; {  utypes[u].is_naval_mobile = n;  }
+void set_u_is_air_mobile(u, n) int u, n; {  utypes[u].is_air_mobile = n;  }
+void set_u_is_advanced(u, n) int u, n; {  utypes[u].is_advanced = n;  }
+void set_u_is_colonizer(u, n) int u, n; {  utypes[u].is_colonizer = n;  }
+void set_u_is_facility(u, n) int u, n; {  utypes[u].is_facility = n;  }
+	
 int bhwtab = -1;
 int bfwtab = -1;
 int bcwtab = -1;
