@@ -1963,7 +1963,6 @@ ImageFamily *
 get_generic_images(char *name)
 {
     FILE *fp;
-    int found = FALSE;
     ImageFamily *imf;
     LibraryPath *path;
 
@@ -1974,7 +1973,9 @@ get_generic_images(char *name)
       imf = (*imf_interp_hook)(imf, NULL, FALSE);
     if (imf_load_hook != NULL)
       imf = (*imf_load_hook)(imf);
-    if (imf->numsizes == 0) {
+    /* Always check that the imf location is loaded (fixes emblem loading from saved 
+    games on the mac). */
+    if (imf->numsizes == 0 || (imf->location && imf->location->loaded != TRUE)) {
 	/* Maybe collect the names/locations of all image families. */
 	if (!imf_dir_loaded) {
 	    /* (should let name be decided by platform?) */
@@ -1996,13 +1997,14 @@ get_generic_images(char *name)
 	    for_all_library_paths(path) {
 		make_pathname(path->path, imf->location->name, "", spbuf);
 		if (load_imf_file(spbuf, NULL)) {
-		  found = TRUE;
-		  break;
+			imf->location->loaded = TRUE;
+			break;
 		}
 	    }
 	    /* Maybe try the plain filename, just in case. */
-	    if (!found)
-	      load_imf_file(imf->location->name, NULL);
+	    if (!imf->location->loaded) {
+	    	imf->location->loaded = load_imf_file(imf->location->name, NULL);
+	    }
 	    /* We don't complain here about not finding the file, because
 	       we'll get a more useful warning alert later on. */
 	    if (imf_interp_hook != NULL)
