@@ -1,5 +1,5 @@
 /* Manipulate Xconq image format files.
-   Copyright (C) 1995 Stanley T. Shebs.
+   Copyright (C) 1995, 1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -10,8 +10,6 @@ any later version.  See the file COPYING.  */
 #include "misc.h"
 #include "lisp.h"
 #include "imf.h"
-extern int numimages;
-extern ImageFamily **images;
 
 char *indirname;
 
@@ -23,19 +21,16 @@ int showprogress = 0;
 
 int explode = 0;
 
-char *imflib = "";
 char *xconqlib = "";
 
 char spbuf[BUFSIZE];
 
 char readerrbuf[1000];
 
-static void usage PARAMS ((void));
+static void usage(void);
 
 int
-main(argc, argv)
-int argc;
-char *argv[];
+main(int argc, char *argv[])
 {
     int i, rslt;
     char *arg;
@@ -130,7 +125,7 @@ char *argv[];
 }
 
 static void
-usage()
+usage(void)
 {
     fprintf(stderr,
 	    "usage: imf2imf [ -i indir ] [ -o outdir ] [ -p ] [ files ... ]\n");
@@ -138,13 +133,12 @@ usage()
 }
 
 void
-close_displays()
+close_displays(void)
 {
 }
 
 int
-write_entire_game_state(fname)
-char *fname;
+write_entire_game_state(char *fname)
 {
     return 0;
 }
@@ -157,17 +151,14 @@ announce_read_progress()
 }
 
 void  
-syntax_error(x, msg)
-Obj *x;
-char *msg;
+syntax_error(Obj *x, char *msg)
 {
     sprintlisp(readerrbuf, x, BUFSIZE);
     init_warning("syntax error in `%s' - %s", readerrbuf, msg);
 }
 
 void
-low_init_error(str)
-char *str;
+low_init_error(char *str)
 {
     fprintf(stderr, "Error: %s.\n", str);
     fflush(stderr);
@@ -176,8 +167,7 @@ char *str;
 /* A warning just gets displayed, no other action is taken. */
 
 void
-low_init_warning(str)
-char *str;
+low_init_warning(char *str)
 {
     fprintf(stdout, "Warning: %s.\n", str);
     fflush(stdout);
@@ -186,8 +176,7 @@ char *str;
 /* A run error is fatal. */
 
 void
-low_run_error(str)
-char *str;
+low_run_error(char *str)
 {
     fprintf(stderr, "Error: %s.\n", str);
     fflush(stderr);
@@ -195,8 +184,7 @@ char *str;
 }
 
 void
-low_run_warning(str)
-char *str;
+low_run_warning(char *str)
 {
     fprintf(stdout, "Warning: %s.\n", str);
     fflush(stdout);
@@ -205,8 +193,7 @@ char *str;
 /* Fake definitions of unneeded routines called by lisp.c. */
 
 int
-keyword_code(str)
-char *str;
+keyword_code(char *str)
 {
     run_warning("fake keyword_code being called");
     return 0;
@@ -227,34 +214,24 @@ struct a_key {
 };
 
 char *
-keyword_name(k)
-enum keywords k;
+keyword_name(enum keywords k)
 {
     return keywordtable[k].name;
 }
 
-int
-keyword_value(k)
-enum keywords k;
-{
-    run_warning("fake keyword_value being called");
-    return 0;
-}
-
 void
-init_predefined_symbols()
+init_predefined_symbols(void)
 {
 }
 
 int
-lazy_bind(sym)
-Obj *sym;
+lazy_bind(Obj *sym)
 {
     run_warning("fake lazy_bind being called");
     return FALSE;
 }
 
 void
-prealloc_debug()
+prealloc_debug(void)
 {
 }
