@@ -1,5 +1,5 @@
 /* Mac-specific code for Xconq kernel.
-   Copyright (C) 1992-1997 Stanley T. Shebs.
+   Copyright (C) 1992-1997, 1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,19 +16,20 @@ any later version.  See the file COPYING.  */
 #include "lisp.h"
 #include "module.h"
 #include "system.h"
-void mac_abort PARAMS ((void));
+void mac_abort(void);
 
 /* Following are from tp.h, but tp.h is too high-level to include. */
 
+#if 0  /* shouldn't be here */
 extern int numremotes;
 extern int my_rid;
 extern int master_rid;
-extern int tmprid;
+#endif
 
-extern void init_file_port PARAMS ((int hosting));
-extern void close_file_port PARAMS ((void));
-extern void low_file_send PARAMS ((int id, char *buf));
-extern int low_file_receive PARAMS ((int *id, char *buf, int maxchars, int timeout));
+extern void init_file_port(int willhost);
+extern void close_file_port(void);
+extern void low_file_send(int id, char *buf);
+extern int low_file_receive(int *id, char *buf, int maxchars, int timeout);
 
 extern int serial_port_dialog(void);
 
@@ -45,12 +46,30 @@ static void close_appletalk(void);
 #endif /* THINK_C */
 
 #ifdef MPW
+#ifdef NEW_HEADERS
+#include <MacTypes.h>
+#else
 #include <Types.h>
+#endif /* NEW_HEADERS */
 #include <Resources.h>
 #include <Events.h>  /* for TickCount */
 #include <TextUtils.h>  /* for GetIndString */
 #endif /* MPW */
 
+#ifdef __MWERKS__
+#ifdef NEW_HEADERS
+#include <MacTypes.h>
+#else
+#include <Types.h>
+#endif /* NEW_HEADERS */
+#include <Resources.h>
+#include <Events.h>  /* for TickCount */
+#include <TextUtils.h>  /* for GetIndString */
+#include <Quickdraw.h>
+#include <StandardFile.h>
+#include <Devices.h>
+#endif /* __MWERKS__ */
+
 #include <signal.h>
 
 /* We need this in order to find string resources that have filenames
@@ -93,12 +112,8 @@ extern CursHandle current_cursor;
 static void low_serial_send(int id, char *buf);
 static int low_serial_readchars(char *buf, int maxchars, int timeout);
 
-char *connection_method_name;
-
 int connection_method;
 
-int hosting;
-
 /* The HFS volume that the program started with. */
 
 short initialvrefnum;
@@ -109,16 +124,29 @@ static char *checkpoint_fname;
 static char *error_save_fname;
 static char *statistics_fname;
 
+/* We normally keep the library and image folders in the same folder
+   as the app. */
+
 #ifndef XCONQLIB
 #define XCONQLIB ":lib"
 #endif
 
+#ifndef XCONQIMAGES
+#define XCONQIMAGES ":images"
+#endif
+
 char *
-default_library_filename()
+default_library_pathname()
 {
     return XCONQLIB;
 }
 
+char *
+default_images_pathname()
+{
+    return XCONQIMAGES;
+}
+
 char *
 news_filename()
 {
@@ -153,8 +181,10 @@ saved_game_filename()
     return save_fname;
 }
 
+/* (should make use of turn number in checkpoint filename) */
+
 char *
-checkpoint_filename()
+checkpoint_filename(int n)
 {
     Str255 tmpstr;
     char tmpbuf[255];
@@ -216,6 +246,71 @@ statistics_filename()
     return statistics_fname;
 }
 
+/* The purpose of this wrapper for fopen is to handle cases where the 
+mac is opening a unix file and automatic linefeed conversion therefore
+should be disabled. */
+
+FILE *
+open_file(char *filename, char *mode)
+{
+	FILE 	*fp;
+	short	curvrefnum;
+	int 	mac_linefeeds = 0;
+	int 	unix_linefeeds = 0;
+	int	ch;
+
+	/* Hack. Force debug output to top directory. */ 
+	if (strcmp(filename, "Xconq.DebugOut") == 0)
+	    SetVol(NULL, initialvrefnum);
+
+	/* Hack. Force warning output to top directory. */ 
+	if (strcmp(filename, "Xconq.Warnings") == 0)
+	    SetVol(NULL, initialvrefnum);
+
+	/* Writing or Appending: always mac format. */	
+	if (strcmp(mode, "w") == 0) {
+		fp = fopen(filename, "w");
+		return fp;
+	}		
+	if (strcmp(mode, "a") == 0) {
+		fp = fopen(filename, "a");
+		return fp;
+	}		
+	/* Reading: first open in binary mode (no conversion). */
+	fp = fopen(filename, "rb");
+	/* Look in xconq start directory if we did not find a file. */
+	if (fp == NULL) {
+		GetVol(NULL, &curvrefnum);
+		SetVol(NULL, initialvrefnum);
+		fp = fopen(filename, "rb");
+		SetVol(NULL, curvrefnum);
+	}
+	/* Give up if we found nothing there either. */
+	if (fp == NULL)
+	    return fp;
+	/* Binary (image) files don't need conversion. */
+	if (strcmp(mode, "rb") == 0)
+	    return fp;
+	/* Text file. Count the number of linefeeds of each kind. */
+	while ((ch = getc(fp)) != EOF) {
+		if (ch == '\r')
+			++mac_linefeeds;
+		if (ch == '\n')
+			++unix_linefeeds;
+	}
+	/* Close the file (also needed in unix case to reset file pointer). */
+	fclose(fp);
+	/* Assume that a mac file has more mac than unix linefeeds. */
+	if (mac_linefeeds > unix_linefeeds) {
+		/* Reopen mac file in text mode (with conversion). */
+		fp = fopen(filename, "r");
+	} else	{
+		/* Reopen unix file in binary mode (no conversion). */
+		fp = fopen(filename, "rb");
+	}
+	return fp;		
+}
+
 /* Attempt to open a library file. */
 
 FILE *
@@ -233,22 +328,12 @@ open_module_library_file(Module *module)
     for_all_library_paths(p) {
 	make_pathname(p->path, module->name, "g", fullnamebuf);
 	/* Now try to open the file. */
-	fp = fopen(fullnamebuf, "r");
+	fp = open_file(fullnamebuf, "r");
 	if (fp != NULL) {
 	    /* Remember the filename where we found it. */
 	    module->filename = copy_string(fullnamebuf);
 	    return fp;
 	}
-	/* Try the same name in a different directory. */
-	GetVol(NULL, &curvrefnum);
-	SetVol(NULL, initialvrefnum);
-	fp = fopen(fullnamebuf, "r");
-	SetVol(NULL, curvrefnum);
-	if (fp != NULL) {
-	    /* Remember the filename (what about volume?) where we found it. */
-	    module->filename = copy_string(fullnamebuf);
-	    return fp;
-	}
     }
     return NULL;
 }
@@ -267,13 +352,7 @@ open_module_explicit_file(Module *module)
 	    for_all_library_paths(p) {
 		make_pathname(p->path, module->name, "g", fullnamebuf);
 		/* Now try to open the file. */
-		fp = fopen(fullnamebuf, "r");
-		if (fp != NULL)
-		  return fp;
-		GetVol(NULL, &curvrefnum);
-		SetVol(NULL, initialvrefnum);
-		fp = fopen(fullnamebuf, "r");
-		SetVol(NULL, curvrefnum);
+		fp = open_file(fullnamebuf, "r");
 		if (fp != NULL)
 		  return fp;
 	    }
@@ -281,26 +360,21 @@ open_module_explicit_file(Module *module)
     } else {
 	/* Try some other random ideas. */
 	sprintf(fullnamebuf, "%s", module->filename);
-	fp = fopen(module->filename, "r");
+	fp = open_file(module->filename, "r");
 	if (fp != NULL) {
 	    add_library_path("");
 	    return fp;
 	}
 	sprintf(fullnamebuf, ":%s", module->filename);
-	fp = fopen(fullnamebuf, "r");
+	fp = open_file(fullnamebuf, "r");
 	if (fp != NULL)
 	  return fp;
 	sprintf(fullnamebuf, "%s%s", ":lib:", module->filename);
-	fp = fopen(fullnamebuf, "r");
+	fp = open_file(fullnamebuf, "r");
 	if (fp != NULL)
 	  return fp;
-	/* Try opening a library module under where the program started. */
-	GetVol(NULL, &curvrefnum);
-	SetVol(NULL, initialvrefnum);
-	fp = fopen(fullnamebuf, "r");
-	SetVol(NULL, curvrefnum);
     }
-    return fp;
+    return NULL;
 }
 
 FILE *
@@ -312,21 +386,14 @@ open_library_file(char *filename)
     FILE *fp;
 	
     /* Now try to open the file. */
-    fp = fopen(filename, "r");
+    fp = open_file(filename, "r");
     if (fp != NULL)
       return fp;
     /* Generate library pathname. */
     for_all_library_paths(p) {
 	make_pathname(p->path, filename, NULL, fullnamebuf);
 	/* Now try to open the file. */
-	fp = fopen(fullnamebuf, "r");
-	if (fp != NULL)
-	  return fp;
-	/* Change to volume where the program started. */
-	GetVol(NULL, &curvrefnum);
-	SetVol(NULL, initialvrefnum);
-	fp = fopen(fullnamebuf, "r");
-	SetVol(NULL, curvrefnum);
+ 	fp = open_file(fullnamebuf, "r");
 	if (fp != NULL)
 	  return fp;
     }
@@ -344,7 +411,7 @@ open_scorefile_for_reading(char *name)
     SetVol(NULL, initialvrefnum);
     /* Now try to open the file. */
     sprintf(buf, ":Scores:%s", name);
-    fp = fopen(buf, "r");
+    fp = open_file(buf, "r");
     SetVol(NULL, curvrefnum);
     return fp;
 }
@@ -359,8 +426,8 @@ open_scorefile_for_writing(char *name)
     GetVol(NULL, &curvrefnum);
     SetVol(NULL, initialvrefnum);
     /* Now try to open the file. */
-    sprintf(buf, ":Scores:%s", name);
-    fp = fopen(buf, "a");
+    sprintf(buf, "%s", name);
+    fp = open_file(buf, "a");
     SetVol(NULL, curvrefnum);
     return fp;
 }
@@ -395,12 +462,13 @@ make_pathname(char *path, char *name, char *extn, char *pathbuf)
     }
 }
 
-/* Remove a saved game from the system. */
+/* Remove a given file. */
 
-void
-remove_saved_game()
+int
+remove_file(char *fname)
 {
     /* (should implement) */
+    return FALSE;
 }
 
 void
@@ -461,8 +529,8 @@ mac_abort ()
     exit(1);
 }
 
-void
-open_remote_connection(char *methodname)
+int
+open_remote_connection(char *methodname, int willhost)
 {
     int serial_port, rslt;
 
@@ -491,13 +559,15 @@ open_remote_connection(char *methodname)
 	init_serial_port(serial_port);
 	/* Serial supports exactly two programs, so we can easily
 	   set up all the rids right now. */
+#if 0 /* but that would be wrong */
 	master_rid = 1;
-	if (hosting) {
+	if (willhost) {
 	    my_rid = 1;
 	} else {
 	    my_rid = 2;
 	}
 	numremotes = 2;
+#endif
 	break;
       case 2:
 	open_appletalk();
@@ -506,11 +576,12 @@ open_remote_connection(char *methodname)
 	init_warning("No TCP/IP support yet, ignoring");
 	break;
       case 4:
-	init_file_port(hosting);
+	init_file_port(willhost);
 	break;
       default:
 	break;
     }
+    return connection_method;
 }
 
 extern short ser_input_refnum;
@@ -565,12 +636,12 @@ low_receive(int *idp, char *buf, int maxchars, int timeout)
 	break;
       case 1:
 	/* Only one remote if using serial. */
-	*idp = (my_rid == 1 ? 2 : 1);
+	*idp = (0 /*my_rid == 1 */? 2 : 1);
 	rslt = low_serial_readchars(buf, maxchars, timeout);
 	break;
       case 2:
 	/* Only one remote for now. */
-	*idp = (my_rid == 1 ? 2 : 1);
+	*idp = (0 /*my_rid == 1*/ ? 2 : 1);
 	rslt = low_appletalk_receive(idp, buf, maxchars, timeout);
 	break;
       case 3:
@@ -592,7 +663,7 @@ low_receive(int *idp, char *buf, int maxchars, int timeout)
 }
 
 void
-close_remote_connection()
+close_remote_connection(int rid)
 {
     switch (connection_method) {
       case 0:
@@ -842,13 +913,15 @@ open_appletalk()
 
     numgotten = tmpMPPPB.NBP.parm.Lookup.numGotten;
     if (numgotten > 0) {
+#if 0
 	master_rid = 1;
-	if (hosting) {
+	if (willhost) {
 	    my_rid = 1;
 	} else {
 	    my_rid = 2;
 	}
 	numremotes = 1;
+#endif
 	/* (should use this loop to get actual number of remotes) */
 	for (i = 0; i < numgotten; ++i) {
 	    err = NBPExtract(namesbuffer, numgotten, i + 1, &entityname, &remote_addr); 
