@@ -1,5 +1,5 @@
 /* The movement-related actions of Xconq.
-   Copyright (C) 1987-1989, 1991-1998 Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -7,12 +7,16 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
-extern int carryable PARAMS ((int u));
-extern int accelerable PARAMS ((int u));
-extern int accelerator PARAMS ((int u1, int u2));
+extern int carryable(int u);
+extern int accelerable(int u);
+extern int accelerator(int u1, int u2);
+extern int border_slide_possible(int u2, int ox, int oy, int nx, int ny); 
 #include "kernel.h"
 #include "kpublic.h"
 
+int unit_traverse_blockable_by(Unit *unit, Unit *unit2);
+int any_friendly_at(Unit *unit, int x, int y);
+
 enum {
     over_nothing = 0,
     over_own = 1,
@@ -25,7 +29,7 @@ enum {
 
 #undef  DEF_ACTION
 #define DEF_ACTION(name,code,args,prepfn,netprepfn,DOFN,checkfn,ARGDECL,doc)  \
-  extern int DOFN PARAMS (ARGDECL);
+  extern int DOFN ARGDECL;
 
 #include "action.def"
 
@@ -37,9 +41,7 @@ extern int retreating_from;
 /* Record a move action as the next to do. */
 
 int
-prep_move_action(unit, unit2, x, y, z)
-Unit *unit, *unit2;
-int x, y, z;
+prep_move_action(Unit *unit, Unit *unit2, int x, int y, int z)
 {
     if (unit == NULL || unit->act == NULL || unit2 == NULL)
       return FALSE;
@@ -52,9 +54,7 @@ int x, y, z;
 }
 
 int
-net_prep_move_action(unit, unit2, x, y, z)
-Unit *unit, *unit2;
-int x, y, z;
+net_prep_move_action(Unit *unit, Unit *unit2, int x, int y, int z)
 {
     int rslt;
 
@@ -69,40 +69,39 @@ int x, y, z;
    task- and plan-level behavior. */
 
 int
-do_move_action(unit, unit2, nx, ny, nz)
-Unit *unit, *unit2;
-int nx, ny, nz;
+do_move_action(Unit *unit, Unit *unit2, int x, int y, int z)
 {
-    int u, u2, nu2, t, rslt, speed, mpcost, acpcost, ox, oy, oz, ot, dist;
+    int u, u2, t, rslt, speed, mpcost, acpcost, ox, oy, oz, ot, dist;
     int nummoves, dirs[NUMDIRS], numdirs, i, ix, iy, ndist;
 
     u = unit->type;  u2 = unit2->type;
-    t = terrain_at(nx, ny);
+    t = terrain_at(x, y);
     ox = unit2->x;  oy = unit2->y;  oz = unit2->z;
     ot = terrain_at(ox, oy);
     speed = 100;
     mpcost = 1;
-    dist = distance(ox, oy, nx, ny);
+    dist = distance(ox, oy, x, y);
     if (dist == 0) {
 	/* Change of altitude within same cell and/or transport departure. */
-	if (unit->transport != NULL && nz == unit->z) {
-	    mpcost = move_unit(unit2, nx, ny);
+	if (unit->transport != NULL && z == unit->z) {
+	    mpcost = move_unit(unit2, x, y);
 	} else {
-	    unit2->z = nz;
+	    unit2->z = z;
 	    /* (should do more stuff - LOS view might have changed) */
 	}
 	acpcost = 1;
 	nummoves = 1;
 	rslt = A_ANY_DONE;
-    } else if (dist == 1) {
+    } else if (dist == 1
+	       || (dist == 2 && border_slide_possible(u2, ox, oy, x, y))) {
 	/* Movement to an adjacent cell. */
-	if (!inside_area(nx, ny)) {
+	if (!inside_area(x, y)) {
 	    kill_unit(unit2, H_UNIT_LEFT_WORLD);
 	    rslt = A_ANY_DONE;
-	} else if (ut_vanishes_on(u2, t) && !can_move_via_conn(unit2, nx, ny)) {
+	} else if (ut_vanishes_on(u2, t) && !can_move_via_conn(unit2, x, y)) {
 	    kill_unit(unit2, H_UNIT_VANISHED);
 	    rslt = A_MOVE_UNIT_GONE;
-	} else if (ut_wrecks_on(u2, t) && !can_move_via_conn(unit2, nx, ny)) {
+	} else if (ut_wrecks_on(u2, t) && !can_move_via_conn(unit2, x, y)) {
 	    if (u_wrecked_type(u2) == NONUTYPE) {
 		/* Occupants always die if the wrecked unit disappears. */
 		kill_unit(unit, H_UNIT_WRECKED);
@@ -111,20 +110,21 @@ int nx, ny, nz;
 		   if the unit will vanish, so that occupants can escape if
 		   allowed for. */
 		if (!ut_vanishes_on(u_wrecked_type(u2), t)) {
-		    speed = unit_speed(unit2, nx, ny);
-		    mpcost = move_unit(unit2, nx, ny);
+		    speed = unit_speed(unit2, x, y);
+		    mpcost = move_unit(unit2, x, y);
 		}
 		wreck_unit(unit2);
 		/* Now make it go away, taking unlucky occupants with. */
-		if (ut_vanishes_on(nu2, t)) {
+		if (ut_vanishes_on(u2, t)) {
 		    kill_unit(unit2, H_UNIT_VANISHED);
 		}
 	    }
 	    rslt = A_MOVE_UNIT_GONE;
+#if 0 /* seems better just to prevent via move costs... */
 	} else if (ut_vanishes_on(u2, ot)
 		   && (unit->transport == NULL
 		       || uu_ferry_on_leave(unit->transport->type, u2) < over_own)
-		   && !can_move_via_conn(unit2, nx, ny)) {
+		   && !can_move_via_conn(unit2, x, y)) {
 	    /* This case is if the unit is already on a connection in
 	       hostile terrain and tries to move to hospitable terrain
 	       without using a bridge, or if it is in a transport that
@@ -134,7 +134,7 @@ int nx, ny, nz;
 	} else if (ut_wrecks_on(u2, ot)
 		   && (unit->transport == NULL
 		       || uu_ferry_on_leave(unit->transport->type, u2) < over_own)
-		   && !can_move_via_conn(unit2, nx, ny)) {
+		   && !can_move_via_conn(unit2, x, y)) {
 	    if (u_wrecked_type(u2) == NONUTYPE) {
 		/* Occupants always die if the wrecked unit disappears. */
 		kill_unit(unit, H_UNIT_WRECKED);
@@ -144,14 +144,15 @@ int nx, ny, nz;
 		   allowed for. */
 		wreck_unit(unit2);
 		/* Now make it go away, taking unlucky occupants with. */
-		if (ut_vanishes_on(nu2, ot)) {
+		if (ut_vanishes_on(u2, t)) {
 		    kill_unit(unit2, H_UNIT_VANISHED);
 		}
 	    }
 	    rslt = A_MOVE_UNIT_GONE;
+#endif
 	} else {
-	    speed = unit_speed(unit2, nx, ny);
-	    mpcost = move_unit(unit2, nx, ny);
+	    speed = unit_speed(unit2, x, y);
+	    mpcost = move_unit(unit2, x, y);
 	    /* ZOC move cost is added after action is done. */
 	    mpcost += zoc_move_cost(unit2, ox, oy, oz);
 	    rslt = A_ANY_DONE;
@@ -172,7 +173,7 @@ int nx, ny, nz;
 	acpcost = 0;
 	nummoves = 0;
 	while (dist > 0) {
-	    numdirs = choose_move_dirs(unit2, nx, ny, TRUE,
+	    numdirs = choose_move_dirs(unit2, x, y, TRUE,
 				       plausible_move_dir, sort_directions, dirs);
 	    if (numdirs == 0)
 	      break;
@@ -193,7 +194,7 @@ int nx, ny, nz;
 		    break;
 		}
 	    }
-	    ndist = distance(unit2->x, unit2->y, nx, ny);
+	    ndist = distance(unit2->x, unit2->y, x, y);
 	    if (ndist >= dist)
 	      break;
 	    dist = ndist;
@@ -215,19 +216,17 @@ int nx, ny, nz;
 }
 
 int
-check_move_action(unit, unit2, nx, ny, nz)
-Unit *unit, *unit2;
-int nx, ny, nz;
+check_move_action(Unit *unit, Unit *unit2, int x, int y, int z)
 {
     int u, u2, u3, ox, oy, oz, acp, acpavail, mpavail, totcost, m, speed;
-    int maxmpavail;
+    int dist, maxmpavail;
 
     if (!in_play(unit))
       return A_ANY_ERROR;
     if (!in_play(unit2))
       return A_ANY_ERROR;
     /* Note that edge cell dests (used to leave the world) are allowed. */
-    if (!in_area(nx, ny))
+    if (!in_area(x, y))
       return A_ANY_ERROR;
     u = unit->type;  u2 = unit2->type;
     ox = unit2->x;  oy = unit2->y;  oz = unit2->z;
@@ -251,21 +250,27 @@ int nx, ny, nz;
     if (!has_supply_to_act(unit2))
       return A_ANY_NO_MATERIAL;
     /* Destination is outside the world and we're not allowed to leave. */
-    if (!inside_area(nx, ny) && u_mp_to_leave_world(u2) < 0)
+    if (!inside_area(x, y) && u_mp_to_leave_world(u2) < 0)
 	return A_MOVE_CANNOT_LEAVE_WORLD;
     /* Check if the destination is within our move range. */
-    /* (also check for and maybe allow border slides here) */
-    if (distance(ox, oy, nx, ny) > u_move_range(u2))
+    dist = distance(ox, oy, x, y);
+    if ((dist == 2 && !border_slide_possible(u, ox, oy, x, y))
+	&& dist > u_move_range(u2))
       return A_ANY_TOO_FAR;
-    if (nz > 0)
+    /* Disallow nonzero altitudes for now. */
+    if (z > 0)
       return A_ANY_TOO_FAR;
     /* Check if the destination is in a blocking ZOC. */
-    if (in_blocking_zoc(unit, nx, ny, nz))
-      return A_ANY_ERROR;
+    if (in_blocking_zoc(unit, x, y, z))
+      return A_MOVE_BLOCKING_ZOC;
     /* Now start looking at the move costs. */
     u3 = (unit2->transport ? unit2->transport->type : NONUTYPE);
-    totcost = total_move_cost(u2, u3, ox, oy, oz, nx, ny, nz);
-    speed = unit_speed(unit2, nx, ny);
+    totcost = total_move_cost(u2, u3, ox, oy, oz, x, y, z);
+    speed = unit_speed(unit2, x, y);
+    /* A unit with an adjusted speed of zero cannot move except within
+       a cell. */
+    if (speed == 0 && !(ox == x && oy == y && oz == z))
+      return A_ANY_CANNOT_DO;
     /* Check if unit can possibly have enough mp for the move. */
     maxmpavail =
       (((u_acp_max(u) > 0 ? u_acp_max(u) : u_acp(u)) - u_acp_min(u)) * speed)
@@ -279,17 +284,17 @@ int nx, ny, nz;
     if (u_acp_min(u) < 0)
       mpavail = ((unit->act->acp - u_acp_min(u)) * speed) / 100;
     /* Zero mp always disallows movement, unless intra-cell. */
-    if (mpavail <= 0 && !(ox == nx && oy == ny && oz == nz))
+    if (mpavail <= 0 && !(ox == x && oy == y && oz == z))
       return A_MOVE_NO_MP;
     /* The free mp might get us enough moves, so add it before comparing. */
     if (mpavail + u_free_mp(u2) < totcost)
       return A_MOVE_NO_MP;
     /* If destination is too small or already full, we can't move into it. */
-    if ((nz & 1) == 0) {
-	if (!can_occupy_cell(unit2, nx, ny))
+    if ((z & 1) == 0) {
+	if (!can_occupy_cell(unit2, x, y))
 	  return A_MOVE_DEST_FULL;
     } else {
-	if (!can_occupy_conn(unit2, nx, ny, nz))
+	if (!can_occupy_conn(unit2, x, y, z))
 	  return A_MOVE_DEST_FULL;
     }
     /* We have to have a minimum level of supply to be able to move. */
@@ -301,33 +306,71 @@ int nx, ny, nz;
 }
 
 int
-can_move_via_conn(unit, nx, ny)
-Unit *unit;
-int nx, ny;
+can_move_via_conn(Unit *unit, int nx, int ny)
 {
     int c, dir;
 
     if (numconntypes == 0)
       return FALSE;
-    for_all_terrain_types(c) {
-	    if (t_is_connection(c)
-		&& aux_terrain_defined(c)
-		&& (dir = closest_dir(nx - unit->x, ny - unit->y)) >= 0
-		&& connection_at(unit->x, unit->y, dir, c)
-		&& !ut_vanishes_on(unit->type, c)
-		&& !ut_wrecks_on(unit->type, c)) {
-		return TRUE;
-	    }
+    for_all_connection_types(c) {
+	if (aux_terrain_defined(c)
+	    && (dir = closest_dir(nx - unit->x, ny - unit->y)) >= 0
+	    && connection_at(unit->x, unit->y, dir, c)
+	    && !ut_vanishes_on(unit->type, c)
+	    && !ut_wrecks_on(unit->type, c))
+	  return TRUE;
     }
     return FALSE;
 }
 
 int
-unit_speed(unit, nx, ny)
-Unit *unit;
-int nx, ny;
+border_slide_possible(int u, int ox, int oy, int nx, int ny)
 {
-    int u = unit->type, speed, x = unit->x, y = unit->y, angle, windval;
+    int dx, dy, dir, bx, by, bdir, b;
+
+    dx = nx - ox;  dy = ny - oy;
+    /* Pick the cell on the right-hand side of the border in question.
+       Since the distance is 2, and border sliding only works for
+       cells on a "diagonal", choose the dir case-by-case. */
+    if (dx == -2 && dy == 1) {
+	dir = NORTHWEST;
+    } else if (dx == -1) {
+	if (dy == -1) {
+	    dir = WEST;
+	} else if (dy == 2) {
+	    dir = NORTHEAST;
+	} else {
+	    return FALSE;
+	}
+    } else if (dx == 1) {
+	if (dy == -2) {
+	    dir = SOUTHWEST;
+	} else if (dy == 1) {
+	    dir = EAST;
+	} else {
+	    return FALSE;
+	}
+    } else if (dx == 2 && dy == -1) {
+	dir = SOUTHEAST;
+    } else {
+	return FALSE;
+    }
+    bdir = left_dir(left_dir(dir));
+    point_in_dir(ox, oy, dir, &bx, &by);
+    for_all_border_types(b) {
+	if (aux_terrain_defined(b)
+	    && border_at(bx, by, bdir, b)
+	    && ut_mp_to_traverse(u, b) >= 0)
+	  return TRUE;
+    }
+    return FALSE;
+}
+
+int
+unit_speed(Unit *unit, int nx, int ny)
+{
+    int u = unit->type, speed, x = unit->x, y = unit->y;
+    int dir, angle1, windval1, angle2, windval2;
     int occeff, totocceff, totoccdenom;
     Unit *occ;
     
@@ -335,10 +378,17 @@ int nx, ny;
     if (unit->hp < u_hp_max(u) && u_speed_damage_effect(u) != lispnil) {
 	speed = damaged_value(unit, u_speed_damage_effect(u), speed);
     }
+    /* The speed effect due to wind is an average of the wind effect
+       in the current cell and in the new cell. */
     if (winds_defined() && u_speed_wind_effect(u) != lispnil) {
-	angle = angle_with(closest_dir(nx - x, nx - y), wind_dir_at(x, y));
-	windval = wind_value(unit, angle, wind_force_at(x, y), u_speed_wind_effect(u), 10000);
-	speed = (speed * windval) / 100;
+	dir = closest_dir(nx - x, ny - y);
+	angle1 = angle_with(dir, wind_dir_at(x, y));
+	windval1 = wind_value(unit, angle1, wind_force_at(x, y),
+			      u_speed_wind_effect(u), 10000);
+	angle2 = angle_with(dir, wind_dir_at(nx, ny));
+	windval2 = wind_value(unit, angle2, wind_force_at(nx, ny),
+			      u_speed_wind_effect(u), 10000);
+	speed = (speed * ((windval1 + windval2) / 2)) / 100;
     }
     if (unit->occupant /* and any occupant speed effects */) {
         totocceff = 100;
@@ -360,19 +410,17 @@ int nx, ny;
     return speed;
 }
 
-/* Compute and return value for a damaged unit, using a list of (hp val) pairs
-   and interpolating between them. */
+/* Compute and return value for a damaged unit, using a list of
+   (hp val) pairs and interpolating between them. */
 
 int
-damaged_value(unit, effect, maxval)
-Unit *unit;
-Obj *effect;
-int maxval;
+damaged_value(Unit *unit, Obj *effect, int maxval)
 {
     int u, err, rslt;
 
     u = unit->type;
-    err = interpolate_in_list_ext(unit->hp, effect, 0, 0, 0, 0, u_hp(u), maxval, &rslt);
+    err = interpolate_in_list_ext(unit->hp, effect, 0, 0, 0, 0, u_hp(u),
+				  maxval, &rslt);
     if (err != 0) {
 	run_warning("cannot get damaged speed for %s at hp %d, using 100",
 		    u_type_name(u), unit->hp);
@@ -385,15 +433,12 @@ int maxval;
    and interpolating between them. */
 
 int
-wind_value(unit, angle, force, effect, maxval)
-Unit *unit;
-int angle, force, maxval;
-Obj *effect;
+wind_value(Unit *unit, int angle, int force, Obj *effect, int maxval)
 {
     int err, rslt;
     Obj *rest, *head, *key, *val;
 
-    for (rest = effect; rest != lispnil; rest = cdr(rest)) {
+    for_all_list(effect, rest) {
 	head = car(rest);
 	key = car(head);
 	if ((numberp(key) && angle == c_number(key))
@@ -407,7 +452,8 @@ Obj *effect;
 		if (err == 0) {
 		    return rslt;
 		} else {
-		    run_warning("no value for wind angle=%d force=%d", angle, force);
+		    run_warning("no value for wind angle=%d force=%d",
+				angle, force);
 		    return maxval;
 		}
 	    }
@@ -417,9 +463,7 @@ Obj *effect;
 }
 
 int
-number_member(x, lis)
-int x;
-Obj *lis;
+number_member(int x, Obj *lis)
 {
     Obj *rest;
 
@@ -442,9 +486,7 @@ Obj *lis;
    in the same cell. */
 
 int
-move_unit(unit, nx, ny)
-Unit *unit;
-int nx, ny;
+move_unit(Unit *unit, int nx, int ny)
 {
     int u = unit->type, u3, ox = unit->x, oy = unit->y, oz = unit->z;
     int nz = oz;
@@ -485,56 +527,57 @@ int nx, ny;
 	    change_control_side_around(nx, ny, u, unit->side);
 	}
     }
+    kick_out_enemy_users(unit->side, nx, ny);
+    
     /* Use up supplies as directed. */
     consume_move_supplies(unit);
     /* a hack */
-    update_cell_display(unit->side, ox, oy, TRUE);
+    update_cell_display(unit->side, ox, oy, UPDATE_ALWAYS);
     /* Always return the mp cost, even if the mover died. */
     return total_move_cost(u, u3, ox, oy, oz, nx, ny, nz);
 }
 
+/* Given a location, change the controlling side. */
+
 void
-change_control_side_around(x, y, u, side)
-int x, y, u;
-Side *side;
+change_control_side_around(int x, int y, int u, Side *side)
 {
-    int con = control_side_at(x, y), s = side_number(side), dir, x1, y1;
+    int con = control_side_at(x, y), s = side_number(side);
     Side *oldside, *side2;
 
-    if (con != NOCONTROL
-        && con != s
-        && !trusted_side(side, side_n(con))) {
-        oldside = side_n(con);
-	set_control_side_at(x, y, s);
-	if (side) {
-	    for_all_sides(side2) {
-		if (side == side2 || trusted_side(side, side2)) {
-		    add_cover(side2, x, y, 1);
-		}
-	    }
+    /* Do nothing if the cell cannot be controlled by a side. */
+    if (con == NOCONTROL)
+      return;
+    /* Do nothing if we control this cell already. */
+    if (con == s)
+      return;
+    oldside = side_n(con);
+    /* Don't challenge control by our allies. */
+    if (trusted_side(side, oldside))
+      return;
+    /* Change the cell's control. */
+    set_control_side_at(x, y, s);
+    /* All of our buddies get to see what goes on here. */
+    for_all_sides(side2) {
+	if (side == side2 || trusted_side(side, side2)) {
+	    add_cover(side2, x, y, 1);
 	}
-	update_cell_display_all_sides(x, y, TRUE);
-	for_all_directions(dir) {
-	    if (interior_point_in_dir(x, y, dir, &x1, &y1)) {
-		update_cell_display_all_sides(x1, y1, TRUE);
-	    }
-	}
-	/* Previous side(s) lose free coverage. */
-	for_all_sides(side2) {
-	    if (!trusted_side(side, side2)
-		&& (oldside == side2 || trusted_side(oldside, side2))) {
-		add_cover(side2, x, y, -1);
-		/* Update coverage display. */
-		update_cell_display(side2, x, y, 36);
-	   }
+    }
+    update_cell_display_all_sides(x, y, UPDATE_ALWAYS | UPDATE_ADJ);
+    /* Previous side(s) lose free coverage. */
+    for_all_sides(side2) {
+	if (!trusted_side(side, side2)
+	    && (oldside == side2 || trusted_side(oldside, side2))) {
+	    add_cover(side2, x, y, -1);
+	    /* Update coverage display. */
+	    update_cell_display(side2, x, y, UPDATE_COVER);
 	}
     }
     /* (should add ability to change adjacent cells also) */
 }
 
 int
-can_move_at_all(unit)
-Unit *unit;
+can_move_at_all(Unit *unit)
 {
     return u_speed(unit->type) > 0;
 }
@@ -543,14 +586,12 @@ Unit *unit;
 
 static int tmprslt;
 
-static void test_blocking_zoc PARAMS ((int x, int y));
+static void test_blocking_zoc(int x, int y);
 
 int
-in_blocking_zoc(unit, x, y, z)
-Unit *unit;
-int x, y, z;
+in_blocking_zoc(Unit *unit, int x, int y, int z)
 {
-    int u = unit->type, t = terrain_at(x, y), dir, x1, y1, u2, range;
+    int u = unit->type, t = terrain_at(x, y), dir, x1, y1, t1, u2, range;
     Unit *unit2;
 
     if (max_zoc_range < 0)
@@ -571,14 +612,25 @@ int x, y, z;
     if (max_zoc_range >= 1) {
 	for_all_directions(dir) {
 	    if (point_in_dir(x, y, dir, &x1, &y1)) {
-		for_all_stack(x, y, unit2) {
+		for_all_stack(x1, y1, unit2) {
 		    u2 = unit2->type;
 		    range = zoc_range(unit2, u);
+		    t1 = terrain_at(x1, y1);
 		    if (range >= 1
 			&& is_active(unit2)
 			&& unit_blockable_by(unit, unit2)
 			&& ut_zoc_into(u2, t)
-			&& ut_zoc_from_terrain(u2, terrain_at(x1, y1)) > 0)
+			&& ut_zoc_from_terrain(u2, t1) > 0)
+		      return TRUE;
+		    if (range >= 1
+			&& is_active(unit2)
+			&& unit_traverse_blockable_by(unit, unit2)
+			&& distance(unit->x, unit->y, x1, y1) <= range
+			/* (should check that zoc goes into old terrain) */
+			&& ut_zoc_into(u2, t)
+			&& ut_zoc_from_terrain(u2, t1) > 0
+			&& !any_friendly_at(unit, x, y)
+			)
 		      return TRUE;
 		}
 	    }
@@ -594,8 +646,7 @@ int x, y, z;
 }
 
 static void
-test_blocking_zoc(x, y)
-int x, y;
+test_blocking_zoc(int x, int y)
 {
     int u = tmpunit->type, u2, t = terrain_at(x, y), range;
     Unit *unit2;
@@ -615,21 +666,44 @@ int x, y;
 /* This is true if unit2 wants to block unit from moving. */
 
 int
-unit_blockable_by(unit, unit2)
-Unit *unit, *unit2;
+unit_blockable_by(Unit *unit, Unit *unit2)
 {
-    return (!trusted_side(unit->side, unit2->side) /* should make a better test */
+    /* should make a better test than just trust - some types
+       might not need to side relationship? */
+    return (!trusted_side(unit->side, unit2->side)
 	    && uu_mp_to_enter_zoc(unit->type, unit2->type) < 0);
 }
 
+/* This is true if unit2 wants to block unit from moving. */
+
+int
+unit_traverse_blockable_by(Unit *unit, Unit *unit2)
+{
+    /* should make a better test than just trust - some types
+       might not need to side relationship? */
+    return (!trusted_side(unit->side, unit2->side)
+	    && uu_mp_to_traverse_zoc(unit->type, unit2->type) < 0);
+}
+
+int
+any_friendly_at(Unit *unit, int x, int y)
+{
+    Unit *unit2;
+
+    for_all_stack(x, y, unit2) {
+	if (unit_trusts_unit(unit, unit2))
+	  return TRUE;
+    }
+    return FALSE;
+}
+
 /* Compute the number of move points that will be needed to do the given
    move. */
 
 int
-total_move_cost(u, u2, x1, y1, z1, x2, y2, z2)
-int u, u2, x1, y1, z1, x2, y2, z2;
+total_move_cost(int u, int u2, int x1, int y1, int z1, int x2, int y2, int z2)
 {
-    int cost, ferry, b, c, conncost, dist, dir;
+    int cost, ferry, b, c, conncost, travcost, dist, dir;
 
     if (z1 != 0 || z2 != 0) {
     	/* should write these calcs eventually */
@@ -661,14 +735,23 @@ int u, u2, x1, y1, z1, x2, y2, z2;
     }
     if (ferry < over_own) {
     	cost += ut_mp_to_leave(u, terrain_at(x1, y1));
+	/* Add in effect of any coatings at the origin. */
+	if (numcoattypes > 0) {
+	    for_all_terrain_types(c) {
+		if (t_is_coating(c)
+		    && aux_terrain_defined(c)
+		    && aux_terrain_at(x1, y1, c) > 0) {
+		    cost += ut_mp_to_leave(u, c);
+		}
+	    }
+	}
     }
     if (numbordtypes > 0 && ferry < over_border) {
 	/* Add any necessary border crossing costs. */
 	dir = closest_dir(x2 - x1, y2 - y1);
 	if (dir >= 0) {
-	    for_all_terrain_types(b) {
-		if (t_is_border(b)
-		    && aux_terrain_defined(b)
+	    for_all_border_types(b) {
+		if (aux_terrain_defined(b)
 		    && border_at(x1, y1, dir, b)) {
 		    cost += ut_mp_to_enter(u, b);
 		}
@@ -677,6 +760,16 @@ int u, u2, x1, y1, z1, x2, y2, z2;
     }
     if (ferry < over_all) {
 	cost += ut_mp_to_enter(u, terrain_at(x2, y2));
+	/* Add in effect of any coatings at the destination. */
+	if (numcoattypes > 0) {
+	    for_all_terrain_types(c) {
+		if (t_is_coating(c)
+		    && aux_terrain_defined(c)
+		    && aux_terrain_at(x2, y2, c) > 0) {
+		    cost += ut_mp_to_enter(u, c);
+		}
+	    }
+	}
     }
     /* Use a connection traversal if it would be cheaper.  This is
        only automatic if the connection on/off costs are small enough,
@@ -686,13 +779,13 @@ int u, u2, x1, y1, z1, x2, y2, z2;
 	/* Try each connection type to see if it's better. */
 	dir = closest_dir(x2 - x1, y2 - y1);
 	if (dir >= 0) {
-	    for_all_terrain_types(c) {
-		if (t_is_connection(c)
-		    && aux_terrain_defined(c)
-		    && connection_at(x1, y1, dir, c)) {
+	    for_all_connection_types(c) {
+		if (aux_terrain_defined(c)
+		    && connection_at(x1, y1, dir, c)
+		    && ((travcost = ut_mp_to_traverse(u, c)) >= 0)) {
 		    conncost = ut_mp_to_enter(u, c)
-		      + ut_mp_to_traverse(u, c)
-			+ ut_mp_to_leave(u, c);
+		      + travcost
+		      + ut_mp_to_leave(u, c);
 		    cost = min(cost, conncost);
 		}
 	    }
@@ -721,12 +814,10 @@ int u2;
 
 static int tmpmpcost, tmpox, tmpoy;
 
-static void zoc_cost_fn PARAMS ((int x, int y));
+static void zoc_cost_fn(int x, int y);
 
 int
-zoc_move_cost(unit, ox, oy, oz)
-Unit *unit;
-int ox, oy, oz;
+zoc_move_cost(Unit *unit, int ox, int oy, int oz)
 {
     int u = unit->type, u2, t1, t2, cost, mpcost, x, y, dir, x1, y1, range;
     Unit *unit2;
@@ -826,16 +917,16 @@ int ox, oy, oz;
 }
 
 static int
-allowed_in_zoc(unit, unit2)
-Unit *unit, *unit2;
+allowed_in_zoc(Unit *unit, Unit *unit2)
 {
     /* should make a better test */
     return trusted_side(unit->side, unit2->side);
 }
 
+static void zoc_cost_fn(int x, int y);
+
 static void
-zoc_cost_fn(x, y)
-int x, y;
+zoc_cost_fn(int x, int y)
 {
     int u, u2, x0, y0, dist, odist, range, cost;
     Unit *unit2;
@@ -881,16 +972,15 @@ int x, y;
 /* This is a hook to handle any reactions to the unit's successful move. */
 
 int
-maybe_react_to_move(unit, ox, oy)
-Unit *unit;
-int ox, oy;
+maybe_react_to_move(Unit *unit, int ox, int oy)
 {
     return 0;
 }
 
+static void try_detonate_on_approach(int x, int y);
+
 static void
-try_detonate_on_approach(x, y)
-int x, y;
+try_detonate_on_approach(int x, int y)
 {
     int dist;
     Unit *unit;
@@ -908,8 +998,7 @@ int x, y;
 }
 
 void
-detonate_on_approach_around(unit)
-Unit *unit;
+detonate_on_approach_around(Unit *unit)
 {
     int maxrange;
 
@@ -983,7 +1072,7 @@ int
 do_enter_action(unit, unit2, unit3)
 Unit *unit, *unit2, *unit3;
 {
-    int u2, u3, u4, ox, oy, oz, ot, nx, ny, nz, nu2, speed, acpcost, mpcost, rslt;
+    int u2, u3, u4, ox, oy, oz, ot, nx, ny, nz, speed, acpcost, mpcost, rslt;
 
     u2 = unit2->type;
     ox = unit2->x;  oy = unit2->y;  oz = unit2->z;
@@ -1013,7 +1102,7 @@ Unit *unit, *unit2, *unit3;
 	       allowed for. */
 	    wreck_unit(unit);
 	    /* Now make it go away, taking unlucky occupants with. */
-	    if (ut_vanishes_on(nu2, ot)) {
+	    if (ut_vanishes_on(u2, ot)) {
 		kill_unit(unit2, H_UNIT_VANISHED);
 	    }
 	}
@@ -1108,15 +1197,15 @@ int
 sides_allow_entry(unit, transport)
 Unit *unit, *transport;
 {
-    if (unit->side == NULL) {
-    	if (transport->side == NULL) {
+    if (unit->side == indepside) {
+    	if (transport->side == indepside) {
     	    return TRUE;
     	} else {
 	    /* (should fix this - table does not apply to indeps entering?) */
     	    return uu_can_enter_indep(unit->type, transport->type);
     	}
     } else {
-    	if (transport->side == NULL) {
+    	if (transport->side == indepside) {
     	    return uu_can_enter_indep(unit->type, transport->type);
     	} else {
 	    /* Note that because this is for an explicit action, the unit
@@ -1133,7 +1222,7 @@ int
 total_entry_cost(u1, u3, x1, y1, z1, u2, x2, y2, z2)
 int u1, u3, x1, y1, z1, u2, x2, y2, z2;
 {
-    int cost = 0, ferryout, ferryin, t1, t2, b, dir, conncost, c;
+    int cost = 0, ferryout, ferryin, t1, t2, b, dir, conncost, travcost, c;
 
     ferryout = over_nothing;
     ferryin = uu_ferry_on_enter(u2, u1);
@@ -1152,9 +1241,8 @@ int u1, u3, x1, y1, z1, u2, x2, y2, z2;
     if (numbordtypes > 0 && ferryout < over_border && ferryin < over_border) {
 	dir = closest_dir(x2 - x1, y2 - y1);
 	if (dir >= 0) {
-	    for_all_terrain_types(b) {
-		if (t_is_border(b)
-		    && aux_terrain_defined(b)
+	    for_all_border_types(b) {
+		if (aux_terrain_defined(b)
 		    && border_at(x1, y1, dir, b)) {
 		    cost += ut_mp_to_enter(u1, b);
 		}
@@ -1174,13 +1262,13 @@ int u1, u3, x1, y1, z1, u2, x2, y2, z2;
 	/* Try each connection type to see if it's better. */
 	dir = closest_dir(x2 - x1, y2 - y1);
 	if (dir >= 0) {
-	    for_all_terrain_types(c) {
-		if (t_is_connection(c)
-		    && aux_terrain_defined(c)
-		    && connection_at(x1, y1, dir, c)) {
+	    for_all_connection_types(c) {
+		if (aux_terrain_defined(c)
+		    && connection_at(x1, y1, dir, c)
+		    && ((travcost = ut_mp_to_traverse(u1, c)) >= 0)) {
 		    conncost = ut_mp_to_enter(u1, c)
-		      + ut_mp_to_traverse(u1, c)
-			+ ut_mp_to_leave(u1, c);
+		      + travcost
+		      + ut_mp_to_leave(u1, c);
 		    cost = min(cost, conncost);
 		}
 	    }
