@@ -2629,10 +2629,16 @@ get_emblem_images(Side *side, Side *side2)
 	       && !strchr(side2->colorscheme, ',')) {
 	/* One-color color scheme can be a substitute emblem. */
 	name = side2->colorscheme;
+    	/* Make sure the color emblemnane is set so that it is 
+    	saved correctly. */
+    	net_set_side_emblemname(side, side2, name);
     } else {
 	/* There is a set of default emblems named "s1", etc. */
 	sprintf(tmpbuf, "s%d", s2);
 	name = copy_string(tmpbuf);
+    	/* Make sure the default emblemnane is set so that it is 
+    	saved correctly. */
+    	net_set_side_emblemname(side, side2, name);
     }
     imf = get_generic_images(name);
     /* If we must have an image, and none were acquired, invoke the default
