@@ -1,5 +1,5 @@
 /* Random utilities not classifiable elsewhere.
-   Copyright (C) 1987-1989, 1991-1997 Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1997, 1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -8,8 +8,8 @@ any later version.  See the file COPYING.  */
 
 #include "config.h"
 #include "misc.h"
-extern int select_by_weight PARAMS ((int *arr, int numvals));
-extern void debugging_string PARAMS ((char *buf));
+extern int select_by_weight(int *arr, int numvals);
+extern void debugging_string(char *buf);
 #include "lisp.h"
 #include "dir.h"
 
@@ -21,6 +21,32 @@ extern void debugging_string PARAMS ((char *buf));
 #endif
 #endif
 
+#ifdef MAC
+
+/* Fake definitions for the Mac. */
+
+struct timeval {
+    time_t tv_sec;
+    int tv_usec;
+};
+
+extern int gettimeofday(struct timeval *tv, void *tz);
+
+int gettimeofday(struct timeval *tv, void *tz)
+{
+    int ticks = TickCount();
+
+    tv->tv_sec = ticks / 60;
+    tv->tv_usec = (ticks % 60) * (1000000 / 60);
+    return 0;
+}
+
+#else
+
+#include <sys/time.h>
+
+#endif /* MAC */
+
 /* Direction-to-delta conversions. */
 
 int dirx[NUMDIRS] = DIRX;
@@ -31,6 +57,10 @@ int diry[NUMDIRS] = DIRY;
 
 int warnings_suppressed;
 
+/* If true, errors and warnings will be recorded in a file. */
+
+int warnings_logged = TRUE;
+
 /* If true, allocation attempts will garner complaints, but not actually
    fail.  (This is primarily for debugging addon code, such as new AIs
    and user interfaces.) */
@@ -56,9 +86,9 @@ static char *padbuf;
 
 /* The random state *must* be at least 32 bits. */
 
-long initrandstate = -1;
+int initrandstate = -1;
 
-long randstate = -1;
+int randstate = -1;
 
 /* Seed can come from elsewhere, for repeatability.  Otherwise, it comes
    from the current time, scaled down to where 32-bit arithmetic won't
@@ -80,11 +110,11 @@ int seed;
 	randstate = seed;
     } else {
     	time(&tm);
-    	randstate = (long) tm;
+    	randstate = (int) tm;
     }
     /* Whatever its source, put the randstate into known range (0 - 99999). */
     randstate = ABS(randstate);
-    randstate %= 100000L;
+    randstate %= 100000;
     /* This is kept around for the sake of error reporting. */
     initrandstate = randstate;
 }
@@ -96,8 +126,20 @@ int
 xrandom(m)
 int m;
 {
-    randstate = (8121 * randstate + 28411) % 134456L;
-    return ((m * randstate) / 134456L);
+    int rslt;
+    long long bignum;
+
+    randstate = (8121 * randstate + 28411) % 134456;
+    /* Random numbers in too large of a range will overflow 32-bit
+       arithmetic, but Xconq rarely needs the large ranges, so only do
+       the 64-bit arithmetic if necessary. */
+    if (m <= 10000) {
+	rslt = (m * randstate) / 134456;
+    } else {
+	bignum = m * ((long long) randstate);
+	rslt = bignum / 134456;
+    }
+    return rslt;
 }
 
 /* Percentage probability, with bounds checking. */
@@ -178,8 +220,8 @@ struct a_malloc {
     int count;
 } *mallocs = NULL;
 
-static void record_malloc PARAMS ((int amt));
-static int malloc_record_compare PARAMS ((const void *m1, const void *m2));
+static void record_malloc(int amt);
+static int malloc_record_compare(const void *m1, const void *m2);
 
 /* Given an amount of memory allocated, record it with the others. */
 
@@ -287,7 +329,17 @@ int amt;
       return NULL;
     /* Do the actual allocation. */
     value = (char *) malloc(amt);
+
+#if 1
+    /* Don't die if the only reason why malloc returned NULL is that it was asked to 
+    allocate zero blocks. This hack makes it possible to run games with terrain images 
+    that are incompatible with the mac interface (should eventually fix imaging code 
+    instead). */ 
+    if (value == NULL && amt != 0) {
+#else
     if (value == NULL) {
+#endif
+
 #ifdef DEBUGGING
 	if (Debug) {
 	    Dprintf("Unable to allocate %d bytes.\n", amt);
@@ -315,7 +367,6 @@ int amt;
 
 /* Like sprintf, but appends. */
 
-#ifdef __STDC__
 void
 tprintf(char *buf, char *str, ...)
 {
@@ -351,33 +402,6 @@ vtprintf(char *buf, char *str, va_list ap)
     vsprintf(line, str, ap);
     strcat(buf, line);
 }
-#else
-void
-tprintf(buf, str, a1, a2, a3, a4, a5, a6, a7, a8, a9)
-char *buf, *str;
-long a1, a2, a3, a4, a5, a6, a7, a8, a9;
-{
-    char line[300];
-
-    sprintf(line, str, a1, a2, a3, a4, a5, a6, a7, a8, a9);
-    strcat(buf, line);
-}
-
-void
-tnprintf(buf, n, str, a1, a2, a3, a4, a5, a6, a7, a8, a9)
-char *buf, *str;
-int n;
-long a1, a2, a3, a4, a5, a6, a7, a8, a9;
-{
-    int n1 = n - strlen(buf);
-    char line[300];
-
-    if (n1 > 0) {
-	sprintf(line, str, a1, a2, a3, a4, a5, a6, a7, a8, a9);
-	strncat(buf, line, n1);
-    }
-}
-#endif
 
 int
 select_by_weight(arr, numvals)
@@ -545,7 +569,7 @@ FILE *dmfp = NULL;
 void
 init_debug_to_stdout()
 {
-   	dfp  = stdout;
+    dfp  = stdout;
     dgfp = stdout;
     dmfp = stdout;
 }
@@ -563,25 +587,25 @@ int firstdebug = TRUE;
 void
 update_debugging()
 {
-	if (Debug || DebugG || DebugM) {
-		/* Always close the file if open, forces to desirable state. */
-		if (ffp != NULL) {
-			fclose(ffp);
-			ffp = NULL;
-		}
-		/* Reopen the file. */
-		if (ffp == NULL) {
-			ffp = fopen("Xconq.DebugOut", "a");
-		}
-		if (ffp != NULL) {
-			if (Debug)
-			  dfp = ffp;
-			if (DebugG)
-			  dgfp = ffp;
-			if (DebugM)
-			  dmfp = ffp;
-		}
+    if (Debug || DebugG || DebugM) {
+	/* Always close the file if open, forces to desirable state. */
+	if (ffp != NULL) {
+	    fclose(ffp);
+	    ffp = NULL;
 	}
+	/* Reopen the file. */
+	if (ffp == NULL) {
+	    ffp = open_file("Xconq.DebugOut", "a");
+	}
+	if (ffp != NULL) {
+	    if (Debug)
+	      dfp = ffp;
+	    if (DebugG)
+	      dgfp = ffp;
+	    if (DebugM)
+	      dmfp = ffp;
+	}
+    }
 }
 
 /* Debug output goes to a file. */
@@ -590,63 +614,61 @@ void
 toggle_debugging(flagp)
 int *flagp;
 {
-	/* Always close the file if open, forces to desirable state. */
-	if (ffp != NULL) {
-		fclose(ffp);
-		ffp = NULL;
-	}
-	/* Flip the state of the debugging flag, if supplied. */ 
-	if (flagp != NULL) {
-		*flagp = ! *flagp;
-	}
-	/* (Re-)open the debugging transcript file. */
-	if (ffp == NULL) {
-		ffp = fopen("Xconq.DebugOut", (firstdebug ? "w" : "a"));
-		firstdebug = FALSE;
-	}
-	if (ffp != NULL && flagp != NULL) {
-		/* Indicate which flags are now on. */
-		fprintf(ffp, "\n\n*********** %s %s %s **********\n\n",
-				(Debug ? "Debug" : ""), (DebugM ? "DebugM" : ""),
-				(DebugG ? "DebugG" : ""));
-	}
-	/* Set specific debug file pointers to be the same as the
+    /* Always close the file if open, forces to desirable state. */
+    if (ffp != NULL) {
+	fclose(ffp);
+	ffp = NULL;
+    }
+    /* Flip the state of the debugging flag, if supplied. */ 
+    if (flagp != NULL) {
+	*flagp = ! *flagp;
+    }
+    /* (Re-)open the debugging transcript file. */
+    if (ffp == NULL) {
+	ffp = open_file("Xconq.DebugOut", (firstdebug ? "w" : "a"));
+	firstdebug = FALSE;
+    }
+    if (ffp != NULL && flagp != NULL) {
+	/* Indicate which flags are now on. */
+	fprintf(ffp, "\n\n*********** %s %s %s **********\n\n",
+		(Debug ? "Debug" : ""), (DebugM ? "DebugM" : ""),
+		(DebugG ? "DebugG" : ""));
+    }
+    /* Set specific debug file pointers to be the same as the
 	   pointer to the file. */
+    if (ffp != NULL) {
+	if (Debug)
+	  dfp = ffp;
+	if (DebugG)
+	  dgfp = ffp;
+	if (DebugM)
+	  dmfp = ffp;
+    }
+    /* If all debugging flags have been turned off, close the file too. */
+    if (!Debug && !DebugG && !DebugM) {
 	if (ffp != NULL) {
-		if (Debug)
-		  dfp = ffp;
-		if (DebugG)
-		  dgfp = ffp;
-		if (DebugM)
-		  dmfp = ffp;
+	    fclose(ffp);
+	    ffp = NULL;
 	}
-	/* If all debugging flags have been turned off, close the file too. */
-	if (!Debug && !DebugG && !DebugM) {
-		if (ffp != NULL) {
-			fclose(ffp);
-			ffp = NULL;
-		}
-	}
-	prealloc_debug();
+    }
+    prealloc_debug();
 }
 
 void
 debugging_string(buf)
 char *buf;
 {
-	buf[0] = '\0';
-	if (Debug || DebugM || DebugG)
-	  strcat(buf, "-D");
-	if (!Debug)
-	  strcat(buf, "-");
-	if (DebugM)
-	  strcat(buf, "M");
-	if (DebugG)
-	  strcat(buf, "G");
+    buf[0] = '\0';
+    if (Debug || DebugM || DebugG)
+      strcat(buf, "-D");
+    if (!Debug)
+      strcat(buf, "-");
+    if (DebugM)
+      strcat(buf, "M");
+    if (DebugG)
+      strcat(buf, "G");
 }
 
-#ifdef __STDC__
-
 void
 debug_printf(char *str, ...)
 {
@@ -676,85 +698,159 @@ debugg_printf(char *str, ...)
     vfprintf(dgfp, str, ap);
     va_end(ap);
 }
-#else
-/* Kind of cheesy, should only use if real ANSI not available. */
-
-void
-debug_printf(str, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12)
-char *str;
-long a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12;
-{
-    fprintf(dfp, str, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12);
-}
-
-void
-debugm_printf(str, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12)
-char *str;
-long a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12;
-{
-    fprintf(dmfp, str, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12);
-}
-
-void
-debugg_printf(str, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12)
-char *str;
-long a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12;
-{
-    fprintf(dgfp, str, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12);
-}
-#endif /* __STDC__ */
 
 #endif /* DEBUGGING */
 
+/* The following code records execution traces of game activities.  This
+   is generally only useful for high-level code such as run_game. */
+
+struct a_timeline_entry {
+  int start;
+  int end;
+  char *type;
+  int detail1;
+  int detail2;
+};
+
+#define BLOCKSIZE 10000
+
+typedef struct a_timeline_block {
+  struct a_timeline_entry entries[BLOCKSIZE];
+  int index;
+  struct a_timeline_block *next;
+} TimelineBlock;
+
+int activity_trace;
+
+static int starting_trace = TRUE;
+
+static TimelineBlock *current_timeline_block;
+
+static char *current_activity;
+
+static struct timeval initial_time;
+
 void
-#ifdef __STDC__
-init_error(char *str, ...)
-#else
-init_error(str, a1, a2, a3, a4, a5, a6, a7, a8, a9)
-char *str;
-long a1, a2, a3, a4, a5, a6, a7, a8, a9;
+record_activity_start(char *type, int detail)
+{
+    TimelineBlock *tlblock;
+    int usec;
+    struct timeval curtime;
+
+    if (!activity_trace)
+      return;
+    if (current_activity != NULL)
+      run_warning("starting activity %s before %s done\n",
+		  type, current_activity);
+    /* Allocate a timeline block if needed. */
+    if (current_timeline_block == NULL) {
+	current_timeline_block =
+	  (TimelineBlock *) xmalloc(sizeof(TimelineBlock));
+    }
+    if (starting_trace) {
+	current_timeline_block->index = 0;
+	gettimeofday(&initial_time, NULL);
+	starting_trace = FALSE;
+    }
+    tlblock = current_timeline_block;
+    if (tlblock->index >= (BLOCKSIZE - 2))
+      return;
+    gettimeofday(&curtime, NULL);
+    usec = (curtime.tv_sec - initial_time.tv_sec) * 1000000
+      + curtime.tv_usec - initial_time.tv_usec;
+    tlblock->entries[tlblock->index].start = usec;
+    tlblock->entries[tlblock->index].type = type;
+    tlblock->entries[tlblock->index].detail1 = detail;
+    current_activity = type;
+}
+
+void
+record_activity_end(char *type, int detail)
+{
+    TimelineBlock *tlblock;
+    int usec;
+    struct timeval curtime;
+
+    if (!activity_trace)
+      return;
+    if (type != current_activity)
+      run_warning("ending activity %s instead of %s\n",
+		  type, (current_activity ? current_activity : "<null>"));
+    tlblock = current_timeline_block;
+    if (tlblock->index >= (BLOCKSIZE - 2))
+      return;
+    gettimeofday(&curtime, NULL);
+    usec = (curtime.tv_sec - initial_time.tv_sec) * 1000000
+      + curtime.tv_usec - initial_time.tv_usec;
+    tlblock->entries[tlblock->index].end = usec;
+    tlblock->entries[tlblock->index].detail2 = detail;
+    ++(tlblock->index);
+    current_activity = NULL;
+}
+
+void
+dump_activity_trace(void)
+{
+    int i;
+    FILE *fp;
+    TimelineBlock *tlblock;
+    struct a_timeline_entry *entry;
+
+    tlblock = current_timeline_block;
+    if (tlblock == NULL)
+      return;
+    if ((fp = open_file("xconq.trace", "w")) != NULL) {
+	for (i = 0; i < tlblock->index; ++i) {
+	    entry = &(tlblock->entries[i]);
+	    fprintf(fp, "%d %d %s %d %d\n",
+		    entry->start, entry->end - entry->start, entry->type,
+		    entry->detail1, entry->detail2);
+#if 0  /* doubles size of trace */
+	    if (tlblock->entries[i+1].start > entry->end + 5) {
+		fprintf(fp, "%d %d %s %d %d\n",
+			entry->end + 1,
+			tlblock->entries[i+1].start - entry->end,
+			"gap", 0, 0);
+	    }
 #endif
+	}
+	fclose(fp);
+    }
+    /* Once we've dumped a trace, prepare to start over. */
+    starting_trace = TRUE;
+}
+
+/* Handling for various kinds of failures. */
+
+void
+init_error(char *str, ...)
 {
     char buf[BUFSIZE];
+    va_list ap;
 
-#ifdef __STDC__
-    {
-	va_list ap;
+    va_start(ap, str);
+    vsprintf(buf, str, ap);
+    va_end(ap);
 
-	va_start(ap, str);
-	vsprintf(buf, str, ap);
-	va_end(ap);
-    }
-#else
-    sprintf(buf, str, a1, a2, a3, a4, a5, a6, a7, a8, a9);
-#endif
     Dprintf("INIT ERROR %s INIT ERROR\n", buf);
+    if (warnings_logged)
+      log_warning("INIT ERROR", buf);
     low_init_error(buf);
 }
 
 void
-#ifdef __STDC__
 init_warning(char *str, ...)
-#else
-init_warning(str, a1, a2, a3, a4, a5, a6, a7, a8, a9)
-char *str;
-long a1, a2, a3, a4, a5, a6, a7, a8, a9;
-#endif
 {
     char buf[BUFSIZE];
+    va_list ap;
 
-#ifdef __STDC__
-    {
-	va_list ap;
+    va_start(ap, str);
+    vsprintf(buf, str, ap);
+    va_end(ap);
 
-	va_start(ap, str);
-	vsprintf(buf, str, ap);
-	va_end(ap);
-    }
-#else
-    sprintf(buf, str, a1, a2, a3, a4, a5, a6, a7, a8, a9);
-#endif
     Dprintf("INIT WARNING %s INIT WARNING\n", buf);
+    if (warnings_logged)
+      log_warning("INIT", buf);
     if (warnings_suppressed)
       return;
     low_init_warning(buf);
@@ -763,28 +859,18 @@ long a1, a2, a3, a4, a5, a6, a7, a8, a9;
 /* A run error is fatal. */
 
 void
-#ifdef __STDC__
 run_error(char *str, ...)
-#else
-run_error(str, a1, a2, a3, a4, a5, a6, a7, a8, a9)
-char *str;
-long a1, a2, a3, a4, a5, a6, a7, a8, a9;
-#endif
 {
     char buf[BUFSIZE];
+    va_list ap;
 
-#ifdef __STDC__
-    {
-	va_list ap;
+    va_start(ap, str);
+    vsprintf(buf, str, ap);
+    va_end(ap);
 
-	va_start(ap, str);
-	vsprintf(buf, str, ap);
-	va_end(ap);
-    }
-#else
-    sprintf(buf, str, a1, a2, a3, a4, a5, a6, a7, a8, a9);
-#endif
     Dprintf("RUN ERROR %s RUN ERROR\n", buf);
+    if (warnings_logged)
+      log_warning("RUN ERROR", buf);
     low_run_error(buf);
 }
 
@@ -792,29 +878,41 @@ long a1, a2, a3, a4, a5, a6, a7, a8, a9;
    usually a problem with Xconq or a game design. */
 
 void
-#ifdef __STDC__
 run_warning(char *str, ...)
-#else
-run_warning(str, a1, a2, a3, a4, a5, a6, a7, a8, a9)
-char *str;
-long a1, a2, a3, a4, a5, a6, a7, a8, a9;
-#endif
 {
     char buf[BUFSIZE];
+    va_list ap;
 
-#ifdef __STDC__
-    {
-	va_list ap;
+    va_start(ap, str);
+    vsprintf(buf, str, ap);
+    va_end(ap);
 
-	va_start(ap, str);
-	vsprintf(buf, str, ap);
-	va_end(ap);
-    }
-#else
-    sprintf(buf, str, a1, a2, a3, a4, a5, a6, a7, a8, a9);
-#endif
     Dprintf("RUN WARNING %s RUN WARNING\n", buf);
+    if (warnings_logged)
+      log_warning("RUN", buf);
     if (warnings_suppressed)
       return;
     low_run_warning(buf);
 }
+
+/* When we log a warning, we want to open and close the file each time,
+   just in case the warning immediately precedes disaster. */
+
+void
+log_warning(char *typ, char *str)
+{
+    static int first_warning = TRUE;
+    static FILE *ewfp;
+
+    if (ewfp == NULL) {
+	/* (should make filename platform-specific) */
+	ewfp = open_file("Xconq.Warnings", (first_warning ? "w" : "a"));
+	/* (should record date) */
+    }
+    if (ewfp != NULL) {
+	fprintf(ewfp, "%s: %s\n", typ, str);
+	fclose(ewfp);
+	ewfp = NULL;
+	first_warning = FALSE;
+    }
+}
