@@ -1,5 +1,5 @@
 /* Commands for the Mac interface to Xconq.
-   Copyright (C) 1992-1998 Stanley T. Shebs.
+   Copyright (C) 1992-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -7,27 +7,30 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
-extern void report_take PARAMS ((Side *side, Unit *unit, int needed, short *rslts));
 #include "print.h"
-#include "kpublic.h"
 #include "macconq.h"
-extern void enable_command(void);
-extern void set_construction_run_length(int len);
-extern Map *front_map(void);
-extern void do_idle_command(int n);
-extern int do_down_command(void);
-extern int do_up_command(void);
 
-/* Define in order to get profiling controls. */
+/* Define to enable (but not activate) profiling. */
 #undef PROFILING
 
-#ifdef THINK_C
-#include <profile.h>
+/* Define PROFILING if CodeWarrior profiling is on. 
+Also include Profiler.h if not already included. */
+#if __profile__
+#include <Profiler.h>
+#define PROFILING
 #endif
 
 #ifdef PROFILING
-extern int _trace;
+#ifdef THINK_C			
+extern int _trace;	/* Not used by CodeWarrior. */
+#include <profile.h>
 #endif
+#endif
+
+extern void set_construction_run_length(int len);
+extern void do_idle_command(int n);
+extern int do_down_command(void);
+extern int do_up_command(void);
 
 extern WindowPtr playersetupwin;
 
@@ -53,17 +56,10 @@ Map *modal_map;
 
 extern char *cursavename;
 
-extern int givetakeprefixarg;
+int givetakeprefixarg;
 
 extern Side *tmpcmdside;
 
-#undef DEF_CMD
-#define DEF_CMD(letter,name,args,FN,help) extern void FN(void);
-
-#include "cmd.def"
-
-#include "maccmd.def"
-
 /* Local function declarations. */
 
 static int do_one_add_terrain(Unit *unit);
@@ -71,6 +67,7 @@ static int do_one_ai_control(Unit *unit);
 static int do_one_attack(Unit *unit);
 static void unit_do_build_2(Unit *unit);
 extern int do_one_clear_plan(Unit *unit);
+extern int do_one_collect(Unit *unit);
 static int do_one_delay(Unit *unit);
 extern int do_one_detach(Unit *unit);
 extern int do_one_detonate(Unit *unit);
@@ -94,7 +91,7 @@ static int do_one_set_name(Unit *unit);
 static int do_one_asleep(Unit *unit);
 extern int do_one_take(Unit *unit);
 
-static void cmd_error(Side *side, char *fmt, ...);
+void cmd_error(Side *side, char *fmt, ...);
 
 /* Static (local) variables. */
 
@@ -108,179 +105,36 @@ static Unit *tmpcmdunit;
 /* This is the actual key typed, for use if several keyboard commands
    share a single function. */
 
-static char tmpkey;
+extern char tmpkey;
 
 /* Prefixed numeric argument to commands. */
 
-static int prefixarg = -1;
-
 static int tmpdir;
 
-static char *cmdargstr;
-
 /* This flag is set to prevent running an "other" command when already
    executing an "other" command. */
 
 static int doingother = FALSE;
 
-typedef struct cmdtab {
-    char fchar;                 /* character to match against */
-    char *name;                 /* full name of command */
-    void (*fn)(void);           /* pointer to command's function */
-    char *help;                 /* short documentation string */
-} CmdTab;
-
-#define C(c) ((c)-0x40)
-
-#undef DEF_CMD
-#define DEF_CMD(LETTER,NAME,args,FN,HELP) { LETTER, NAME, FN, HELP },
-
-/* The generic command table. */
-
-CmdTab commands[] = {
-
-#include "cmd.def"
-
-  { 0, NULL, NULL, NULL }
-};
-
-/* The Mac-specific command table. */
-
-CmdTab mac_commands[] = {
-
-#include "maccmd.def"
-
-  { 0, NULL, NULL, NULL }
-};
-
 /* Given a character, find a command for it and execute. */
 
 void
 do_keyboard_command(int key)
 {
-	CmdTab *cmd;
     void (*fn)(void);
 
 	DGprintf("Typed '%c' (0x%x)\n", key, key);
 	if (between('0', key, '9')) {
 		/* Add a decimal digit to the prefix argument. */
-		if (prefixarg < 0)
-		  prefixarg = 0;
-		prefixarg += prefixarg * 10 + (key - '0');
+		if (dside->prefixarg < 0)
+		  dside->prefixarg = 0;
+		dside->prefixarg += dside->prefixarg * 10 + (key - '0');
 		/* (should add some sort of feedback) */
 	} else {
-		/* Look through the generic command table. */
-	    for (cmd = commands; cmd->name != NULL; ++cmd) {
-			if (key == cmd->fchar) {
-		    	if ((fn = cmd->fn) == NULL) {
-					run_warning("no command function for %s (0x%x)?", cmd->name, key);
-					return;
-		    	}
-		    	tmpkey = key;
-				(*fn)();
-				/* Reset the prefix argument. */
-				prefixarg = -1;
-				return;
-			}
-		}
-		/* Look through the Mac-specific command table. */
-	    for (cmd = mac_commands; cmd->name != NULL; ++cmd) {
-			if (key == cmd->fchar) {
-		    	if ((fn = cmd->fn) == NULL) {
-					run_warning("no command function for %s (0x%x)?", cmd->name, key);
-					return;
-		    	}
-		    	tmpkey = key;
-				(*fn)();
-				/* Reset the prefix argument. */
-				prefixarg = -1;
-				return;
-			}
-		}
+		execute_command(dside, key);
 	}
 }
 
-void
-execute_named_command(char *cmdstr)
-{
-	char *cmdname;
-	int prefix;
-	CmdTab *cmd;
-    void (*fn)(void);
-
-    parse_long_name_command(cmdstr, &prefix, &cmdname, &cmdargstr, copy_string(cmdstr)); 
-    if (empty_string(cmdname)) {
-		notify(dside, "No command name.");
-		beep();
-		return;
-	}
-    for (cmd = commands; cmd->name != NULL; ++cmd) {
-		if (strcmp(cmdname, cmd->name) == 0) {
-			fn = cmd->fn;
-	    	if (fn == NULL) {
-				run_warning("no command function for %s?", cmd->name);
-				return;
-	    	}
-	    	/* Set the prefix arg if one was on the command line. */
-	    	if (prefix >= 0)
-	    	  prefixarg = prefix;
-	    	/* Use the command's char as the apparent key. */
-	    	tmpkey = cmd->fchar;
-			(*fn)();
-			/* Reset the prefix argument. */
-			prefixarg = -1;
-			return;
-		}
-	}
-    for (cmd = mac_commands; cmd->name != NULL; ++cmd) {
-		if (strcmp(cmdname, cmd->name) == 0) {
-			fn = cmd->fn;
-	    	if (fn == NULL) {
-				run_warning("no command function for %s?", cmd->name);
-				return;
-	    	}
-	    	/* Use the command's char as the apparent key. */
-	    	tmpkey = cmd->fchar;
-			(*fn)();
-			/* Reset the prefix argument. */
-			prefixarg = -1;
-			return;
-		}
-	}
-	notify(dside, "Command \"%s\" not recognized.", cmdname);
-	beep();
-}
-
-void
-describe_keyboard(int arg, char *key, TextBuffer *buf)
-{
-	CmdTab *cmd;
-
-	tbcat(buf, "Single-key commands:\n\n");
-    for (cmd = commands; cmd->name != NULL; ++cmd) {
-		describe_command(cmd->fchar, cmd->name, cmd->help, TRUE, buf);
-    }
-	tbcat(buf, "\n\nSingle-key commands (Mac-only):\n\n");
-    for (cmd = mac_commands; cmd->name != NULL; ++cmd) {
-		describe_command(cmd->fchar, cmd->name, cmd->help, TRUE, buf);
-    }
-}
-
-void
-describe_text_commands(int arg, char *key, TextBuffer *buf)
-{
-	CmdTab *cmd;
-
-    tbcat(buf, "\nText commands:\n\n");
-    for (cmd = commands; cmd->name != NULL; ++cmd) {
-		describe_command (cmd->fchar, cmd->name, cmd->help, FALSE, buf);
-    }
-    tbcat(buf, "\nText commands (Mac-only):\n\n");
-    for (cmd = mac_commands; cmd->name != NULL; ++cmd) {
-		describe_command (cmd->fchar, cmd->name, cmd->help, FALSE, buf);
-    }
-}
-
 int
 get_a_position(Map **mapp, int *xp, int *yp, int *dirp)
 {
@@ -290,7 +144,8 @@ get_a_position(Map **mapp, int *xp, int *yp, int *dirp)
 		position_set_modally = FALSE;
 		*mapp = modal_map;
 		return m_nearest_boundary(*mapp, modal_point.h, modal_point.v, xp, yp, dirp);
-	} else if ((*mapp = map_from_window(FrontWindow())) != NULL) {
+	} else if ((*mapp = frontmap) != NULL) {
+
 		GetMouse(&at);
 		return m_nearest_boundary(*mapp, at.h, at.v, xp, yp, dirp);
 	}
@@ -306,7 +161,8 @@ get_a_unit(Map **mapp, Unit **unitp)
 		position_set_modally = FALSE;
 		*mapp = modal_map;
 		return m_nearest_unit(*mapp, modal_point.h, modal_point.v, unitp);
-	} else if ((*mapp = map_from_window(FrontWindow())) != NULL) {
+	} else if ((*mapp = frontmap) != NULL) {
+
 		GetMouse(&at);
 		return m_nearest_unit(*mapp, at.h, at.v, unitp);
 	}
@@ -325,7 +181,8 @@ get_a_visible_unit(Map **mapp, Unit **unitp)
 		position_set_modally = FALSE;
 		*mapp = modal_map;
 		rslt = m_nearest_unit(*mapp, modal_point.h, modal_point.v, unitp);
-	} else if ((*mapp = map_from_window(FrontWindow())) != NULL) {
+	} else if ((*mapp = frontmap) != NULL) {
+
 		GetMouse(&at);
 		rslt = m_nearest_unit(*mapp, at.h, at.v, unitp);
 	}
@@ -339,14 +196,16 @@ get_a_visible_unit(Map **mapp, Unit **unitp)
 
 /* Start of alphabetized commands. */
 
-void
-do_add_player()
-{
-    net_request_additional_side(cmdargstr);
-}
+void do_flash(Side *side) {}
+void do_world_map(Side *side) {}
+void do_unit_closeup(Side *side) {}
+void do_side_closeup(Side *side) {}
+void do_orders_popup(Side *side) {}
+void do_new_map(Side *side) {}
+void do_map(Side *side) {}
 
 void
-do_add_terrain()
+do_add_terrain(Side *side)
 {
     int x, y, dir;
 	Map *map;
@@ -377,8 +236,8 @@ do_one_add_terrain(Unit *unit)
 		t1 = possibles[0];
 	} else {
 		/* should have better way to choose type */
-		if (between(0, prefixarg, numtypes - 1))
-		  t1 = possibles[prefixarg];
+		if (between(0, dside->prefixarg, numtypes - 1))
+		  t1 = possibles[dside->prefixarg];
 		else {
 			notify(dside, "Prefix should be 0 to %d", numtypes - 1); 
 			return FALSE;
@@ -413,17 +272,7 @@ do_one_add_terrain(Unit *unit)
 }
 
 void
-do_ai_side()
-{
-	if (side_has_ai(dside)) {
-		net_set_side_ai(dside, NULL);
-	} else {
-		net_set_side_ai(dside, "mplayer");
-	}
-}
-
-void
-do_attack()
+do_attack(Side *side)
 {
 	Map *map;
 	
@@ -457,7 +306,7 @@ do_one_attack(Unit *atker)
 }
 
 void
-do_auto()
+do_auto(Side *side)
 {
 	apply_to_all_selected(do_one_ai_control, TRUE);	
 }
@@ -468,9 +317,9 @@ do_one_ai_control(Unit *unit)
     int newval;
 
 	if (unit->plan) {
-		if (prefixarg < 0)
+		if (dside->prefixarg < 0)
 		  newval = !unit->plan->aicontrol;
-		else if (prefixarg == 0)
+		else if (dside->prefixarg == 0)
 		  newval = 0;
 		else
 		  newval = 1;
@@ -480,13 +329,14 @@ do_one_ai_control(Unit *unit)
 }
 
 void
-do_build()
+do_build(Side *side)
 {
 	int i;
-	Map *map;  List *list;  UnitCloseup *closeup;
+	List *list;  
+	UnitCloseup *closeup;
 	Unit *unit;
 
-	list = list_from_window(FrontWindow());
+	list = list_from_window(topWindow);
 	if (list != NULL) {
 		unit = selected_unit_in_list(list);
 		if (unit != NULL && can_build_or_help(unit)) {
@@ -495,7 +345,7 @@ do_build()
 		}
 		return;
 	}
-	closeup = unit_closeup_from_window(FrontWindow());
+	closeup = unit_closeup_from_window(topFloat);
 	if (closeup != NULL) {
 		unit = closeup->unit;
 		if (unit != NULL && can_build_or_help(unit)) {
@@ -504,10 +354,9 @@ do_build()
 		}
 		return;
 	}
-	map = front_map();
-	if (map != NULL) {
-		for (i = 0; i < map->numselections; ++i) {
-			unit = map->selections[i];
+	if (frontmap) {
+		for (i = 0; i < frontmap->numselections; ++i) {
+			unit = frontmap->selections[i];
 			if (unit != NULL && can_build_or_help(unit)) {
 				unit_do_build_2(unit);
 				return;
@@ -525,14 +374,14 @@ unit_do_build_2(Unit *unit)
     extern int editedrunlength;
 
 	modal_construction = TRUE;
-	window_behind_construction = FrontWindow();
-	if (prefixarg > 0)
-	  set_construction_run_length(prefixarg);
-	if (prefixarg > 0)
-	  editedrunlength = prefixarg;
+	window_behind_construction = topWindow;
+	if (dside->prefixarg > 0)
+	  set_construction_run_length(dside->prefixarg);
+	if (dside->prefixarg > 0)
+	  editedrunlength = dside->prefixarg;
 	else
 	  editedrunlength = -1;
-	enable_construction();
+	show_construction_window();
 	select_unit_in_construction_window(unit);
 	select_type_in_construction_window(favored_type(unit));
 }
@@ -540,17 +389,22 @@ unit_do_build_2(Unit *unit)
 /* Create and/or bring up the construction planning window. */
 
 void
-enable_construction()
+show_construction_window()
 {
-	window_behind_construction = FrontWindow();
+	window_behind_construction = topWindow;
 	if (constructionwin == nil) {
 		create_construction_window();
-	}
-	if (constructionwin != nil) {
+	} else {
 		reinit_construction_lists();
-		ShowWindow(constructionwin);
-		SelectWindow(constructionwin);
 	}
+	SelectTheWindow(constructionwin);
+	ActivateWindow(constructionwin, true);
+}
+
+void
+do_clear_plan(Side *side)
+{
+	apply_to_all_selected(do_one_clear_plan, TRUE);	
 }
 
 int
@@ -564,20 +418,50 @@ do_one_clear_plan(Unit *unit)
 	return FALSE;
 }
 
+int mtocollect;
+
 void
-do_clear_plan()
+do_collect(Side *side)
 {
-	apply_to_all_selected(do_one_clear_plan, TRUE);	
+    char *arg, *rest;
+
+	mtocollect = NONMTYPE;
+	if (nummtypes == 0) {
+		cmd_error(dside, "No materials to collect");
+		return;
+	}
+	if (!empty_string(cmdargstr)) {
+		rest = get_next_arg(cmdargstr, tmpbuf, &arg);
+		mtocollect = mtype_from_name(arg);
+	} else {
+		cmd_error(dside, "No material name given");
+		return;
+	}
+	if (!is_material_type(mtocollect)) {
+		cmd_error(dside, "`%s' is not a recognized material name", arg);
+		return;
+	}
+	apply_to_all_selected(do_one_collect, TRUE);	
+}
+
+int
+do_one_collect(Unit *unit)
+{
+	if (unit->plan) {
+		net_set_collect_task(unit, mtocollect, unit->x, unit->y);
+		return TRUE;
+	}
+	return FALSE;
 }
 
 void
-do_copying()
+do_copying(Side *side)
 {
-	help_dialog(copying_help_node);
+	show_help_window(copying_help_node);
 }
 
 void
-do_delay()
+do_delay(Side *side)
 {
 	apply_to_all_selected(do_one_delay, TRUE);
 }
@@ -593,7 +477,7 @@ do_one_delay(Unit *unit)
 }
 
 void
-do_detach()
+do_detach(Side *side)
 {
 	apply_to_all_selected(do_one_detach, TRUE);
 }
@@ -615,7 +499,7 @@ do_one_detach(Unit *unit)
 /* This command samples the current mouse to get the desired detonate location. */
 
 void
-do_detonate()
+do_detonate(Side *side)
 {
     int x, y, dir;
 	Map *map;
@@ -660,13 +544,11 @@ do_one_dir_move(Unit *unit)
 /* The command function proper. */
 
 void
-do_dir()
+do_dir(Side *side)
 {
 	int ndirs, dir1, dir2, modif;
-	Map *map;
 
-	map = front_map();
-	if (map != NULL) {
+	if (frontmap) {
 		ndirs = char_to_dir(tmpkey, &dir1, &dir2, &modif);
 		if (ndirs >= 1) {
 			tmpdir = dir1;
@@ -675,8 +557,8 @@ do_dir()
 			return;
 		}
 		apply_to_all_selected(do_one_dir_move, TRUE);
-		if (map->moveonclick)
-		  map->scrolltocurunit = TRUE;
+		if (frontmap->moveonclick)
+		  frontmap->scrolltocurunit = TRUE;
 	}
 }
 
@@ -688,7 +570,7 @@ int
 do_one_dir_multiple_move(Unit *unit)
 {
 	if (mobile(unit->type)) {
-		net_set_move_dir_task(unit, tmpdir, (prefixarg <= 0 ? 9999 : prefixarg));
+		net_set_move_dir_task(unit, tmpdir, (dside->prefixarg <= 0 ? 9999 : dside->prefixarg));
 		return TRUE;
 	}
 	return FALSE;
@@ -697,13 +579,11 @@ do_one_dir_multiple_move(Unit *unit)
 /* The command function proper. */
 
 void
-do_dir_multiple()
+do_dir_multiple(Side *side)
 {
 	int ndirs, dir1, dir2, modif;
-	Map *map;
 
-	map = front_map();
-	if (map != NULL) {
+	if (frontmap) {
 		ndirs = char_to_dir(tmpkey, &dir1, &dir2, &modif);
 		if (ndirs >= 1) {
 			tmpdir = dir1;
@@ -712,13 +592,13 @@ do_dir_multiple()
 			return;
 		}
 		apply_to_all_selected(do_one_dir_multiple_move, TRUE);
-		if (map->moveonclick)
-		  map->scrolltocurunit = TRUE;
+		if (frontmap->moveonclick)
+		  frontmap->scrolltocurunit = TRUE;
 	}
 }
 
 void
-do_disband()
+do_disband(Side *side)
 {
 	apply_to_all_selected(do_one_disband, TRUE);
 }
@@ -730,7 +610,7 @@ do_one_disband(Unit *unit)
 }
 
 void
-do_disembark()
+do_disembark(Side *side)
 {
 	apply_to_all_selected(do_one_disembark, TRUE);
 }
@@ -762,56 +642,14 @@ do_one_disembark(Unit *unit)
 }
 
 void
-do_distance()
+do_distance(Side *side)
 {
     notify(dside, "Click, then drag to location to which you want the distance.");
 	query_position_modally(DISTANCE_MODAL);
 }
 
 void
-do_distrust()
-{
-	Side *side2;
-
-	if (cmdargstr) {
-		side2 = parse_side_spec(cmdargstr);
-		if (side2 != NULL && side2 != dside) {
-			net_set_trust(dside, side2, 0);
-			return;
-		} 
-	}
-	beep();
-}
-
-void
-do_doctrine()
-{
-	if (strcmp(cmdargstr, "set") == 0) {
-		notify(dside, "Set what?");
-		beep();
-	} else if (strncmp(cmdargstr, "set ", 4) == 0) {
-		net_set_doctrine(dside, cmdargstr + 4);
-	} else {
-		notify_doctrine(dside, cmdargstr);
-	}
-}
-
-void
-do_down()
-{
-	do_down_command();
-}
-
-void
-do_draw_willingness()
-{
-    if (prefixarg < 0)
-	  prefixarg = 1;
-    net_set_willing_to_draw(dside, (prefixarg ? 1 : 0));
-}
-
-void
-do_embark()
+do_embark(Side *side)
 {
 	apply_to_all_selected(do_one_embark, TRUE);
 }
@@ -874,12 +712,13 @@ do_one_embark(Unit *unit)
 	return FALSE;
 }
 
+#if 0 /* (should fix this) */
 void
-do_end_turn()
+do_end_turn(Side *side)
 {
 	/* <return> is also interpreted by dialogs, so special-case this, depending
 	   on which window was in front. */
-	if (FrontWindow() == constructionwin) {
+	if (topWindow == constructionwin) {
 		Point pt;
 		extern ControlHandle constructbutton;
 		
@@ -890,9 +729,16 @@ do_end_turn()
 		net_finish_turn(dside);
 	}
 }
+#endif
 
 void
-do_fire()
+do_escape(Side *side)
+{
+	map_modal = NO_MODAL;
+}
+
+void
+do_fire(Side *side)
 {
 	Map *map;
 	
@@ -920,7 +766,7 @@ do_one_fire(Unit *atker)
 }
 
 void
-do_fire_into()
+do_fire_into(Side *side)
 {
 	Map *map;
 	
@@ -945,32 +791,23 @@ do_one_fire_into(Unit *atker)
 }
 
 void
-do_follow_action()
+do_follow_action(Side *side)
 {
-	Map *map;
-
-	map = front_map();
-	if (map != NULL) {
-		map->follow_action = !map->follow_action;
-		if (map->follow_action)
-		  notify(dside, "Following the action on map %d", map->id);
+	if (frontmap) {
+		frontmap->follow_action = !frontmap->follow_action;
+		if (frontmap->follow_action)
+		  notify(dside, "Following the action on map %d", frontmap->id);
 		else
-		  notify(dside, "No longer following action on map %d", map->id);
+		  notify(dside, "No longer following action on map %d", frontmap->id);
 	} else {
 		beep();
 	}
 }
 
 void
-do_force_global_replan()
+do_give(Side *side)
 {
-	force_global_replan(dside);
-}
-
-void
-do_give()
-{
-	givetakeprefixarg = prefixarg;
+	givetakeprefixarg = side->prefixarg;
 	apply_to_all_selected(do_one_give, TRUE);
 }
 
@@ -982,13 +819,13 @@ do_one_give(Unit *unit)
 }
 
 void
-do_give_unit()
+do_give_unit(Side *side)
 {
 	int sn = 0;
 
-    if (prefixarg >= 0) {
-		if (prefixarg <= numsides) {
-			sn = prefixarg;
+    if (side->prefixarg >= 0) {
+		if (side->prefixarg <= numsides) {
+			sn = side->prefixarg;
 		} else {
 			beep();
 			return;
@@ -1018,17 +855,17 @@ do_one_give_unit(Unit *unit)
 }
 
 void
-do_help()
+do_help(Side *side)
 {
     /* Bring up the help window at wherever it was, if already
        created, otherwise will start at topics node. */
-	help_dialog(NULL);
+	show_help_window(NULL);
 }
 
 void
-do_idle()
+do_idle(Side *side)
 {
-	do_idle_command(prefixarg);
+	do_idle_command(side->prefixarg);
 }
 
 void
@@ -1046,9 +883,9 @@ do_one_idle(Unit *unit)
 }
 
 void
-do_message()
+do_message(Side *side)
 {
-	message_dialog(prefixarg);
+	message_dialog(side->prefixarg);
 }
 
 /* Dialog for the input of a textual message. */
@@ -1110,7 +947,7 @@ message_dialog(int arg)
 }
 
 void
-do_move_to()
+do_move_to(Side *side)
 {
     notify(dside, "Click on location to which you want to move.");
 	query_position_modally(MOVE_TO_MODAL);
@@ -1139,12 +976,12 @@ do_one_move_to(Unit *unit)
 		return TRUE;
 	}
 #endif /* DESIGNERS */
-	net_set_move_to_task(unit, tmpcmdx, tmpcmdy);
+	net_set_move_to_task(unit, tmpcmdx, tmpcmdy, 0);
 	return TRUE;
 }
 
 void
-do_name()
+do_name(Side *side)
 {
 	apply_to_all_selected(do_one_set_name, TRUE);
 }
@@ -1159,23 +996,21 @@ int
 do_one_occupant(Unit *unit)
 {
 	Unit *nextocc;
-	Map *map;
 
 	nextocc = find_next_occupant(unit);
 	if (nextocc != unit) {
-		map = map_from_window(FrontWindow());
-		if (map != NULL) {
-			unselect_unit_on_map(map, unit);
-			select_unit_on_map(map, nextocc);
+		if (frontmap) {
+			unselect_unit_on_map(frontmap, unit);
+			select_unit_on_map(frontmap, nextocc);
 			/* Overkill, but draw_row alone not as desirable? */
-			update_cell_display(dside, unit->x, unit->y, TRUE);
+			update_cell_display(dside, unit->x, unit->y, UPDATE_ALWAYS);
 		}
 	}
 	return TRUE;
 }
 
 void
-do_occupant()
+do_occupant(Side *side)
 {
 	apply_to_all_selected(do_one_occupant, TRUE);
 }
@@ -1183,13 +1018,13 @@ do_occupant()
 int other_command_prefixarg;
 
 void
-do_other()
+do_other(Side *side)
 {
 	/* Don't allow recursion with this command. */
 	if (!doingother) {
 		doingother = TRUE;
-		other_command_prefixarg = prefixarg;
-		enable_command();
+		other_command_prefixarg = side->prefixarg;
+		show_command_window();
 		other_command_prefixarg = -1;
 		doingother = FALSE;
 	} else {
@@ -1207,37 +1042,35 @@ TEHandle command_text = nil;
 
 Rect commandtextrect;
 
-void create_command_dialog(void);
+void create_command_window(void);
 
 static void get_command_and_do(void);
 
 void
-enable_command()
+show_command_window()
 {
 	if (commandwin == nil) {
-		create_command_dialog();
-	}
-	if (commandwin != nil) {
-		ShowWindow(commandwin);
-		SelectWindow(commandwin);
+		create_command_window();
 	}
+	SelectTheWindow(commandwin);
+	ActivateWindow(commandwin, true);
 }
 
 void
-create_command_dialog()
+create_command_window()
 {
 	Rect tmprect;
 
 	if (hasColorQD) {
-		commandwin = GetNewCWindow(wCommand, NULL, (WindowPtr) -1L);
+		commandwin = GetNewCWindow(wCommand, NULL, NULL);
 	} else {
-		commandwin = GetNewWindow(wCommand, NULL, (WindowPtr) -1L);
+		commandwin = GetNewWindow(wCommand, NULL, NULL);
 	}
 	dobutton = GetNewControl(cCommandDoButton, commandwin);
 	SetPort(commandwin);
 	tmprect = commandwin->portRect;
 	commandtextrect = tmprect;
-	commandtextrect.bottom -= 30;
+	commandtextrect.bottom -= 25;
 	InsetRect(&commandtextrect, 5, 5);
 	/* Create and clear the TextEdit record. */
 	command_text = TENew(&commandtextrect, &commandtextrect);
@@ -1248,7 +1081,7 @@ create_command_dialog()
 		sprintf(spbuf, "%d ", other_command_prefixarg);
 		TEInsert(spbuf, strlen(spbuf), command_text);
 	}
-	ShowWindow(commandwin);
+	SelectTheWindow(commandwin);
 }
 
 void
@@ -1291,6 +1124,7 @@ do_mouse_down_command(Point mouse, int mods)
 
 	part = FindControl(mouse, commandwin, &control);
 	if (control == dobutton) {
+		TrackControl(control, mouse, NULL);
 		get_command_and_do();
 	} else if (PtInRect(mouse, &commandtextrect)) {
 		TEClick(mouse, mods, command_text);
@@ -1309,7 +1143,7 @@ get_command_and_do()
 	strncpy(buffer, *text, len);
 	buffer[len] = '\0';
 	if (!empty_string(buffer)) {
-		execute_named_command(buffer);
+		execute_long_command(dside, buffer);
 	} else {
 		notify(dside, "No command.");
 		beep();
@@ -1317,7 +1151,7 @@ get_command_and_do()
 }
 
 void
-do_print_view()
+do_print_view(Side *side)
 {
 	dump_ps_view(dside, NULL, "View PS");
 }
@@ -1325,14 +1159,15 @@ do_print_view()
 static void unit_do_produce_2(Unit *unit);
 
 void
-do_produce()
+do_produce(Side *side)
 {
 	int i;
-	Map *map;  List *list;  UnitCloseup *closeup;
+	List *list;  
+	UnitCloseup *closeup;
 	Unit *unit;
 
 	/* Try different alternatives to find a unit. */
-	list = list_from_window(FrontWindow());
+	list = list_from_window(topWindow);
 	if (list != NULL) {
 		unit = selected_unit_in_list(list);
 		if (unit != NULL && can_produce(unit)) {
@@ -1341,7 +1176,7 @@ do_produce()
 		}
 		return;
 	}
-	closeup = unit_closeup_from_window(FrontWindow());
+	closeup = unit_closeup_from_window(topFloat);
 	if (closeup != NULL) {
 		unit = closeup->unit;
 		if (unit != NULL && can_produce(unit)) {
@@ -1350,10 +1185,9 @@ do_produce()
 		}
 		return;
 	}
-	map = front_map();
-	if (map != NULL) {
-		for (i = 0; i < map->numselections; ++i) {
-			unit = map->selections[i];
+	if (frontmap) {
+		for (i = 0; i < frontmap->numselections; ++i) {
+			unit = frontmap->selections[i];
 			if (unit != NULL && can_produce(unit)) {
 				unit_do_produce_2(unit);
 				return;
@@ -1371,8 +1205,8 @@ unit_do_produce_2(Unit *unit)
 	int m, n;
 
     n = 9999;
-    if (prefixarg > 0)
-	  n = prefixarg;
+    if (dside->prefixarg > 0)
+	  n = dside->prefixarg;
 	for_all_material_types(m) {
 		if (um_acp_to_produce(unit->type, m) > 0) {
 			net_push_produce_task(unit, m, n);
@@ -1382,7 +1216,7 @@ unit_do_produce_2(Unit *unit)
 }
 
 void
-do_quit()
+do_quit(Side *side)
 {
 	quit_the_game();
 }
@@ -1398,15 +1232,11 @@ add_unit_position(Unit *unit)
 }
 
 void
-do_recenter()
+do_recenter(Side *side)
 {
 	int avgx, avgy;
-	Map *map;
 
-	map = front_map();
-	/* Only apply recentering if the front map window is also
-	   the overall front window. */
-	if (map != NULL && map->window == FrontWindow()) {
+	if (frontmap) {
 		allsumx = allsumy = 0;
 		unitcount = 0;
 		apply_to_all_selected(add_unit_position, FALSE);
@@ -1415,38 +1245,14 @@ do_recenter()
 			return;
 		}
 		avgx = allsumx / unitcount;  avgy = allsumy / unitcount;
-		set_focus(map, avgx, avgy);
+		set_map_focus(frontmap, avgx, avgy);
 	}
 }
 
-
-
-void
-set_focus(Map *map, int x, int y)
-{
-	int	oldsx, oldsy, newsx, newsy;
-
-	if (!inside_area(x, y)) 
-	  return;
-	/* Save the old vp. */
-	oldsx = map->vp->sx;
-	oldsy = map->vp->sy;
-	/* Find the new vp. */
-	set_view_focus(map->vp, x, y);
-	m_center_on_focus(map);
-	newsx = map->vp->sx;
-	newsy = map->vp->sy;
-	/* Restore the old vp. */
-	map->vp->sx = oldsx;
-	map->vp->sy = oldsy;
-	/* Let scroll_map_window do the rest of the updating work. */	
-	scroll_map_window(map, newsx - oldsx, newsy - oldsy);
-}
-
 /* Recalculate and redraw everything. */
 
 void
-do_refresh()
+do_refresh(Side *side)
 {
 	Map *map;
 	List *list;
@@ -1473,22 +1279,7 @@ do_refresh()
 }
 
 void
-do_release()
-{
-	Side *side2;
-
-	if (cmdargstr) {
-		side2 = parse_side_spec(cmdargstr);
-		if (side2 != NULL && side2 != dside) {
-			net_set_controlled_by(side2, dside, FALSE);
-			return;
-		} 
-	}
-	beep();
-}
-
-void
-do_remove_terrain()
+do_remove_terrain(Side *side)
 {
     int x, y, dir;
 	Map *map;
@@ -1528,7 +1319,7 @@ do_one_remove_terrain(Unit *unit)
 }
 
 void
-do_repair()
+do_repair(Side *side)
 {
 	Map *map;
 	
@@ -1574,9 +1365,9 @@ do_reserve_command(int value, int radius, int recurse)
 }
 
 void
-do_reserve()
+do_reserve(Side *side)
 {
-	do_reserve_command(TRUE, prefixarg, FALSE);
+	do_reserve_command(TRUE, side->prefixarg, FALSE);
 }
 
 static int
@@ -1590,33 +1381,33 @@ do_one_return(Unit *unit)
 }
 
 void
-do_return()
+do_return(Side *side)
 {
 	apply_to_all_selected(do_one_return, TRUE);	
 }
 
 void
-do_resign()
+do_resign(Side *side)
 {
 	if (endofgame) {
-		notify(dside, "Game is already over.");
+		notify(side, "Game is already over.");
 		beep();
-	} else if (!dside->ingame) {
-		notify(dside, "You are already out of the game.");
+	} else if (!side->ingame) {
+		notify(side, "You are already out of the game.");
 		beep();
 	} else if (CautionAlert(aConfirmResign, nil) == aiConfirmResignResign) {
-		net_resign_game(dside, NULL);
+		net_resign_game(side, NULL);
 	}
 }
 
 void
-do_save()
+do_save(Side *side)
 {
 	save_the_game(FALSE, FALSE);
 }
 
 void
-do_set_formation()
+do_set_formation(Side *side)
 {
 	int i, numcould = 0, numnot = 0;
 	Map *map;
@@ -1648,24 +1439,32 @@ do_set_formation()
 }
 
 void
-do_set_rate()
+do_set_view_angle(Side *side)
 {
-	int slow, fast;
-	char *reststr;
+	int angle;
 
-	if (cmdargstr) {
-		slow = strtol(cmdargstr, &reststr, 10);
-		fast = strtol(reststr, &reststr, 10);
-		set_play_rate(slow, fast);
-		return;
+	if (frontmap) {
+		angle = frontmap->vp->angle;
+		if (angle == 90)
+		  angle = 30;
+		else if (angle == 30)
+		  angle = 15;
+		else
+		  angle = 90;
+		/* A suitable vertscale is now computed by set_map_angle. 
+		Only override it if provided with a valid prefixarg. */
+		set_map_angle(frontmap, angle);
+		frontmap->vp->vertscale 
+		   = (side->prefixarg < 1 ? frontmap->vp->vertscale : side->prefixarg);
+	} else {
+		beep();
 	}
-	beep();
 }
 
 void
-do_sleep()
+do_sleep(Side *side)
 {
-	do_sleep_command(TRUE, prefixarg, FALSE);
+	do_sleep_command(TRUE, side->prefixarg, FALSE);
 }
 
 void
@@ -1684,7 +1483,7 @@ do_one_asleep(Unit *unit)
 }
 
 void
-do_standing_orders()
+do_standing_orders(Side *side)
 {
 	int rslt;
 
@@ -1697,42 +1496,24 @@ do_standing_orders()
 	beep();
 }
 
-void
-do_submit()
-{
-	Side *side2;
-
-	if (cmdargstr) {
-		side2 = parse_side_spec(cmdargstr);
-		if (side2 != NULL && side2 != dside) {
-			net_set_controlled_by(dside, side2, TRUE);
-			return;
-		} 
-	}
-	beep();
-}
-
 void 
-do_survey()
+do_survey(Side *side)
 {
-	Map *map;
-
-	map = front_map();
-	if (map != NULL) {
-		toggle_survey(map);
+	if (frontmap) {
+		toggle_survey(frontmap);
 	} else {
 		beep();
 	}
 }
 
 void
-do_take()
+do_take(Side *side)
 {
 	if (nummtypes == 0) {
 		beep();
 		return;
 	}
-	givetakeprefixarg = prefixarg;
+	givetakeprefixarg = side->prefixarg;
 	apply_to_all_selected(do_one_take, TRUE);
 }
 
@@ -1750,235 +1531,121 @@ do_one_take(Unit *unit)
 }
 
 void
-do_trust()
-{
-	Side *side2;
-
-	if (cmdargstr) {
-		if (*cmdargstr == '?') {
-			/* (should be an nlang.c function) */
-			char buf[BUFSIZE];
-			int somebody;
-			Side *side3;
-
-			for_all_sides(side2) {
-				sprintf(buf, "%s trust%s", short_side_title(side2),
-						(short_side_title_plural_p(side2) ? "" : "s"));
-				somebody = FALSE;
-				for_all_sides(side3) {
-					if (side3 != side2 && trusted_side(side2, side3)) {
-						strcat(buf, " ");
-						strcat(buf, short_side_title(side3));
-						somebody = TRUE;
-					}
-				}
-				if (!somebody) {
-					strcat(buf, " nobody");
-				}
-				notify(dside, "%s.", buf);
-			}
-			return;
-		}
-		side2 = parse_side_spec(cmdargstr);
-		if (side2 != NULL && side2 != dside) {
-			net_set_trust(dside, side2, 1);
-			return;
-		} 
-	}
-	beep();
-}
-
-void
-do_up()
+do_up(Side *side)
 {
 	do_up_command();
 }
 
 void
-do_version()
+do_version(Side *side)
 {
 	do_about_box();
 }
 
 void
-do_wake()
+do_wake(Side *side)
 {
-	do_sleep_command(FALSE, prefixarg, FALSE);
-	do_reserve_command(FALSE, prefixarg, FALSE);
+	do_sleep_command(FALSE, side->prefixarg, FALSE);
+	do_reserve_command(FALSE, side->prefixarg, FALSE);
 }
 
 void
-do_wake_all()
+do_wake_all(Side *side)
 {
-	do_sleep_command(FALSE, prefixarg, TRUE);
-	do_reserve_command(FALSE, prefixarg, TRUE);
+	do_sleep_command(FALSE, side->prefixarg, TRUE);
+	do_reserve_command(FALSE, side->prefixarg, TRUE);
 }
 
 void
-do_warranty()
+do_warranty(Side *side)
 {
-	help_dialog(warranty_help_node);
+	show_help_window(warranty_help_node);
 }
 
+void
+do_zoom_in(Side *side)
+{
+	if (frontmap) {
+		magnify_map(frontmap, 1);
+	} else {
+		beep();
+	}
+}
+
+void
+do_zoom_out(Side *side)
+{
+	if (frontmap) {
+		magnify_map(frontmap, -1);
+	} else {
+		beep();
+	}
+}
+
+/* End of alphabetized commands. */
+
 #ifdef DESIGNERS
 
 /* Toggle the designer mode. */
 
 void
-do_design()
+do_design(Side *side)
 {
-	if (!dside->designer) {
+	if (!side->designer) {
 		enable_designing(FALSE);
 	} else {
 		disable_designing();
 	}
 }
 
-void
-do_gdl()
-{
-	if (cmdargstr)
-	  interp_form(NULL, read_form_from_string(cmdargstr, NULL, NULL, NULL));
-	else
-	  beep();
-	end_printing_forms();
-}
-
 #endif
 
 #ifdef DEBUGGING
 
 void
-do_debug()
-{
-	toggle_debugging(&Debug);
-	draw_game();
-}
-
-void
-do_debugg()
-{
-	toggle_debugging(&DebugG);
-	draw_game();
-}
-
-void
-do_debugm()
-{
-	toggle_debugging(&DebugM);
-	draw_game();
-}
-
-void
-do_profile()
+do_profile(Side *side)
 {
 	toggle_profiling();
 }
 
 void
-do_trace()
+do_trace(Side *side)
 {
 	toggle_profiling();
+
 #ifdef PROFILING
+
+#ifdef THINK_C			/* Not used by CodeWarrior. */
+
 	_trace = 1;
-#endif
+
+#endif /* THINK_C */
+
+#endif /* PROFILING */
+
 }
 
-#endif
-
-/* End of alphabetized commands. */
-
-/* Mac-specific command functions. */
-
-void
-do_mac_escape()
-{
-	map_modal = NO_MODAL;
-}
-
-void
-do_mac_set_map_angle()
-{
-	int angle;
-	Map *map;
-
-	map = front_map();
-	if (map != NULL) {
-		angle = map->vp->angle;
-		if (angle == 90)
-		  angle = 30;
-		else if (angle == 30)
-		  angle = 15;
-		else
-		  angle = 90;
-		map->vp->vertscale = (prefixarg < 1 ? 1 : prefixarg);
-		/* Erase other-map boxes in other windows */
-		draw_related_maps(map);
-
-		/* Change the view angle */
-		set_view_angle(map->vp, angle);
-
-		/* Update the gworld, but dont show the redrawn map yet */
-		ValidRect(&map->contentrect);
-		BeginUpdate(map->window);		
-		update_gworld(map);
-
-		/* Redraw other-map boxes in other windows */
-		draw_related_maps(map);
-		
-		/* Focus on active unit */
-		do_recenter();
-		
-		/* Now finally copy the result to the screen */
-		EndUpdate(map->window);
-		copy_from_gworld(map, map->contentrect);
-		
-		/* Fix scrollbars if necessary */
-		adjust_map_decor(map);
-	} else {
-		beep();
-	}
-}
-
-void
-do_mac_zoom_in()
-{
-	Map *map;
-
-	map = front_map();
-	if (map != NULL) {
-		magnify_map(map, 1);
-	} else {
-		beep();
-	}
-}
-
-void
-do_mac_zoom_out()
-{
-	Map *map;
-
-	map = front_map();
-	if (map != NULL) {
-		magnify_map(map, -1);
-	} else {
-		beep();
-	}
-}
-
-#ifdef DEBUGGING
-
 void
 toggle_profiling()
 {
-#ifdef PROFILING
-	extern int _profile;
-#endif
 
+#ifdef PROFILING		
+
+#ifdef THINK_C			/* Not used by CodeWarrior. */
+
+	extern int _profile;
+
+#endif /* THINK_C */
+
+	/* Only toggle Profile if profiling is enabled! */
 	Profile = !Profile;
 
-#ifdef PROFILING
+	/* Necessary to update game window with the
+	new profiling status. */
+	draw_game();
+
 #ifdef THINK_C
+
 	if (Profile && !_profile) {
 		InitProfile(1000, 100);
 		freopen("Xconq.ProfileOut", "w", stdout);
@@ -1987,22 +1654,40 @@ toggle_profiling()
 		DumpProfile();
 		fflush(stdout);
 	}
-#endif
-#endif
+
+#endif /* THINK_C */
+
+#if __profile__			/* CodeWarrior support. */
+
+	if (Profile && ProfilerGetStatus() == false) {
+		ProfilerSetStatus(true);
+	}
+	if (!Profile && ProfilerGetStatus() == true) {
+		ProfilerSetStatus(false);
+		ProfilerDump("\pXconq.Profile");
+		ProfilerClear();
+	}
+
+#endif /* __profile__ */
+
+#endif /* PROFILING */
+
 }
 
 #endif /* DEBUGGING */
 
 /* Generic command error feedback. */
 
-static void
+void
 cmd_error(Side *side, char *fmt, ...)
 {
 	va_list ap;
 
-	va_start(ap, fmt);
-    vnotify(side, fmt, ap);
-	va_end(ap);
+	if (!empty_string(fmt)) {
+		va_start(ap, fmt);
+    	vnotify(side, fmt, ap);
+		va_end(ap);
+	}
     /* (should) Only beep once, even if a command generates multiple error messages. */
     beep();
 }
