@@ -734,7 +734,11 @@ adjust_cursor(Point mouse, RgnHandle region)
 	    && PtInRgn(mouse, ((WindowPeek) topFloat)->contRgn) == FALSE) {
 		update_window(topFloat);
 	}
-	map = frontmap;
+	/* Check if we are inside the worldmap window. */
+	if (worldmap && PtInRgn(mouse, ((WindowPeek) worldmap->window)->contRgn))
+		map = worldmap;
+	else	map = frontmap;
+
 	if (map != NULL) {
 		GetPort(&oldport);
 		SetPort(map->window);
@@ -756,28 +760,24 @@ adjust_cursor(Point mouse, RgnHandle region)
 				usual = FALSE;
 				/* Wait half second before autoscrolling. */
 				if (TickCount() > delay + 30)
-					beep();
 					scroll_map_window(map, -4, 0);
 			} else if (mouse.h > map->window->portRect.right - map->sbarwid - 40) {
 				current_cursor = rightcursor;
 				usual = FALSE;
 				/* Wait half second before autoscrolling. */
 				if (TickCount() > delay + 30)
-					beep();
 					scroll_map_window(map, 4, 0);
 			} else if (mouse.v < map->toph + 40) {
 				current_cursor = upcursor;
 				usual = FALSE;
 				/* Wait half second before autoscrolling. */
 				if (TickCount() > delay + 30)
-					beep();
 					scroll_map_window(map, 0, -4);
 			} else if (mouse.v > map->window->portRect.bottom - map->sbarwid - 40) {
 				current_cursor = downcursor;
 				usual = FALSE;
 				/* Wait half second before autoscrolling. */
 				if (TickCount() > delay + 30)
-					beep();
 					scroll_map_window(map, 0, 4);
 			} else	 {
 				/* Reset autoscrolling delay. */
