@@ -175,6 +175,14 @@ typedef pascal void (*UserItemProcPtr)(WindowPtr theWindow, short itemNo);
 #define NewControlActionProc(fn) ((ProcPtr) (fn))
 #endif
 
+/* GetKeys has different declarations for its argument type, depending
+   on random header versions.  This allows local override of the
+   default of long * (some headers have it as unsigned long * ). */
+
+#ifndef GETKEYS_ARG_TYPE
+#define GETKEYS_ARG_TYPE long *
+#endif
+
 enum grays {
 	blackgray,
 	darkgray,
