@@ -344,11 +344,7 @@ any later version.  See the file COPYING.  */
 #define diVariantsPeople 7
 #define diVariantsEconomy 8
 #define diVariantsSupply 9
-#define diVariantsIndepsAI 10
-#define diVariantsIndepsBuild 11
-#define diVariantsIndepsResearch 12
-#define diVariantsIndepsTreasury 13
-#define diVariantsFirstCheckBox 14
+#define diVariantsFirstCheckBox 10
 /* and next 6 also */
 #define diVariantsFirstSlider 20
 /* and next 2 also */
