@@ -423,6 +423,13 @@ draw_side_emblem(WindowPtr win, int ex, int ey, int ew, int eh, int e, int style
 				}
 			}
 
+			/* PlotCIcon seems to have logic for the 1-bit
+			   case that our CopyBits does not, so use it in
+			   this special case only. */
+			if (eimg->pixelsize == 1) {
+				PlotCIcon(&imagerect, cicnhandle);
+				return;
+			}
 			/* Get pixmap, bitmap & maskmap from cicn and set srcrect */
 			pm = (*(cicnhandle))->iconPMap;
 			bm = (*(cicnhandle))->iconBMap;
