@@ -595,7 +595,7 @@ copy_clipped_to_map(Map *map, Rect sourcerect, Rect destrect)
 void
 update_resized_map(Map *map)
 {
-	short right, left, down, up;
+	int right, left, down, up;
 	Rect old_contentrect;
 
 	/* Save and then resize contentrect */
@@ -668,7 +668,7 @@ update_resized_map(Map *map)
 void
 update_gworld(Map *map)
 {
-	short margx, margy, newsx, newsy, oldsx, oldsy;			
+	int margx, margy, newsx, newsy, oldsx, oldsy;			
 	Rect GWportRect;	
 	
 	/* Compute sizes of map margins outside the viewport */
@@ -910,7 +910,7 @@ auto_scroll_left(Map *map)
 
 	while (keyDown) {
 		scroll_map_window(map, -4, 0);
-		GetKeys((unsigned long *) keyMap);
+		GetKeys((GETKEYS_ARG_TYPE) keyMap);
 		keyDown = keyMap[0x56 >> 3] >> (0x56 & 7) & 1;	/* keypad 4 */
 	}
 }
@@ -925,7 +925,7 @@ auto_scroll_right(Map *map)
 
 	while (keyDown) {
 		scroll_map_window(map, 4, 0);
-		GetKeys((unsigned long *) keyMap);
+		GetKeys((GETKEYS_ARG_TYPE) keyMap);
 		keyDown = keyMap[0x58 >> 3] >> (0x58 & 7) & 1;	/* keypad 6 */
 	}
 }
@@ -940,7 +940,7 @@ auto_scroll_up(Map *map)
 
 	while (keyDown) {
 		scroll_map_window(map, 0, -4);
-		GetKeys((unsigned long *) keyMap);
+		GetKeys((GETKEYS_ARG_TYPE) keyMap);
 		keyDown = keyMap[0x5B >> 3] >> (0x5B & 7) & 1;	/* keypad 8 */
 	}
 }
@@ -955,7 +955,7 @@ auto_scroll_down(Map *map)
 
 	while (keyDown) {
 		scroll_map_window(map, 0, 4);
-		GetKeys((unsigned long *) keyMap);
+		GetKeys((GETKEYS_ARG_TYPE) keyMap);
 		keyDown = keyMap[0x54 >> 3] >> (0x54 & 7) & 1;	/* keypad 2 */
 	}
 }
@@ -970,7 +970,7 @@ auto_scroll_down_left(Map *map)
 
 	while (keyDown) {
 		scroll_map_window(map, -4, 4);
-		GetKeys((unsigned long *) keyMap);
+		GetKeys((GETKEYS_ARG_TYPE) keyMap);
 		keyDown = keyMap[0x53 >> 3] >> (0x53 & 7) & 1;	/* keypad 1 */
 	}
 }
@@ -985,7 +985,7 @@ auto_scroll_down_right(Map *map)
 
 	while (keyDown) {
 		scroll_map_window(map, 4, 4);
-		GetKeys((unsigned long *) keyMap);
+		GetKeys((GETKEYS_ARG_TYPE) keyMap);
 		keyDown = keyMap[0x55 >> 3] >> (0x55 & 7) & 1;	/* keypad 3 */
 	}
 }
@@ -1000,7 +1000,7 @@ auto_scroll_up_left(Map *map)
 
 	while (keyDown) {
 		scroll_map_window(map, -4, -4);
-		GetKeys((unsigned long *) keyMap);
+		GetKeys((GETKEYS_ARG_TYPE) keyMap);
 		keyDown = keyMap[0x59 >> 3] >> (0x59 & 7) & 1;	/* keypad 7 */
 	}
 }
@@ -1015,7 +1015,7 @@ auto_scroll_up_right(Map *map)
 
 	while (keyDown) {
 		scroll_map_window(map, 4, -4);
-		GetKeys((unsigned long *) keyMap);
+		GetKeys((GETKEYS_ARG_TYPE) keyMap);
 		keyDown = keyMap[0x5C >> 3] >> (0x5C & 7) & 1;	/* keypad 9 */
 	}
 }
@@ -1030,7 +1030,7 @@ auto_scroll_left_arrow(Map *map)
 
 	while (keyDown) {
 		scroll_map_window(map, -4, 0);
-		GetKeys((unsigned long *) keyMap);
+		GetKeys((GETKEYS_ARG_TYPE) keyMap);
 		keyDown = keyMap[0x7B >> 3] >> (0x7B & 7) & 1;	/* left arrow */
 	}
 }
@@ -1045,7 +1045,7 @@ auto_scroll_right_arrow(Map *map)
 
 	while (keyDown) {
 		scroll_map_window(map, 4, 0);
-		GetKeys((unsigned long *) keyMap);
+		GetKeys((GETKEYS_ARG_TYPE) keyMap);
 		keyDown = keyMap[0x7C >> 3] >> (0x7C & 7) & 1;	/* right arrow */
 	}
 }
@@ -1060,7 +1060,7 @@ auto_scroll_up_arrow(Map *map)
 
 	while (keyDown) {
 		scroll_map_window(map, 0, -4);
-		GetKeys((unsigned long *) keyMap);
+		GetKeys((GETKEYS_ARG_TYPE) keyMap);
 		keyDown = keyMap[0x7E >> 3] >> (0x7E & 7) & 1;	/* keypad 8 */
 	}
 }
@@ -1075,7 +1075,7 @@ auto_scroll_down_arrow(Map *map)
 
 	while (keyDown) {
 		scroll_map_window(map, 0, 4);
-		GetKeys((unsigned long *) keyMap);
+		GetKeys((GETKEYS_ARG_TYPE) keyMap);
 		keyDown = keyMap[0x7D >> 3] >> (0x7D & 7) & 1;	/* keypad 2 */
 	}
 }
