@@ -9,8 +9,8 @@ any later version.  See the file COPYING.  */
 #include "conq.h"
 #include "macconq.h"
 
-static pascal void draw_instructions_text(WindowPtr win, short ditem);
 static pascal void help_vscroll_fn(ControlHandle control, short code);
+static void insert_newlines(TEHandle te, int n);
 
 /* Globals for the instructions window. */
 
@@ -32,11 +32,17 @@ ControlHandle helpvscrollbar;
 
 ControlActionUPP help_vscroll_proc;
 
+/* Used in help window. */
 ControlHandle topicsbutton;
 ControlHandle helpbutton;
 ControlHandle prevbutton;
 ControlHandle nextbutton;
 ControlHandle backbutton;
+ControlHandle closebutton;
+
+/* Used in instructions window. */
+ControlHandle openhelpbutton;
+ControlHandle closewinbutton;
 
 static HelpNode *cur_help_node = NULL;
 
@@ -58,49 +64,17 @@ int nodestackpos;
 
 /* The instructions window. */
 
-static pascal void
-draw_instructions_text(WindowPtr win, short ditem)
-{
-	GrafPtr oldport;
-	short itemtype;  Handle itemhandle;  Rect itemrect;
-
-	GetDItem(instructionswin, diInstructionsText, &itemtype, &itemhandle, &itemrect);
-	GetPort(&oldport);
-	SetPort(instructionswin);
-	TextFont(small_font_id);
-	TextSize(small_font_size);
-	TEUpdate(&itemrect, instructions_text);
-	/* This makes the title item draw big. */
-	TextFont(title_font_id);
-	TextSize(title_font_size);
-	SetPort(oldport);
-}
-
-/* Bring up the dialog with instructions on how to play. */
-
 void
-instructions_dialog()
+show_instructions_window()
 {
-	int mainwidth, mainheight;
-	Rect tmprect;
-
 	if (instructionswin == nil) {
-		create_instructions_dialog();
+		create_instructions_window();
+		draw_instructions();
 	}
-	if (first_windows) {
-		get_main_screen_size(&mainwidth, &mainheight);
-		tmprect = instructionswin->portRect;
-		MoveWindow(instructionswin,
-				   mainwidth - (tmprect.right - tmprect.left) - 3,
-				   mainheight - (tmprect.bottom - tmprect.top) - 3,
-				   FALSE);
-	}
-	ShowWindow(instructionswin);
-	SelectWindow(instructionswin);
+	SelectTheWindow(instructionswin);
+	ActivateWindow(instructionswin, true);
 }
 
-void insert_newlines(TEHandle te, int n);
-
 void
 insert_newlines(TEHandle te, int n)
 {
@@ -109,7 +83,7 @@ insert_newlines(TEHandle te, int n)
 
 	str[0] = '\0';
 	for (i = 0; i < n; ++i) {
-#ifdef THINK_C
+#if defined(THINK_C) || defined(__MWERKS__)
 		strcat(str, "\r");
 #else
 		strcat(str, "\n");
@@ -118,34 +92,64 @@ insert_newlines(TEHandle te, int n)
 	TEInsert(str, strlen(str), te);
 }
 
+/* The non-modal instructions dialog messed up the floating windows, 
+so I have converted it into a real window instead. */
+
 void
-create_instructions_dialog()
+create_instructions_window()
+{
+	/* Create the window, color if possible, since images may be in color. */
+	if (hasColorQD) {	
+		instructionswin = GetNewCWindow(wInstructions, NULL, NULL);
+	} else {
+		instructionswin = GetNewWindow(wInstructions, NULL, NULL);
+	}
+	openhelpbutton = GetNewControl(cHelpButton, instructionswin);
+	MoveControl(openhelpbutton, 175, 200);
+	closewinbutton = GetNewControl(cCloseButton, instructionswin);
+	MoveControl(closewinbutton, 235, 200);
+	add_window_menu_item("Instructions", instructionswin);
+}
+
+/* New function, in part derived from old create_instructions_dialog. */
+
+void
+draw_instructions()
 {
 	int any_instructions = FALSE;
 	Obj *instructions, *rest;
-	char *str;
-	Str255 tmpstr;
-	Rect destrect, viewrect;
-	short itemtype;  Handle itemhandle;  Rect itemrect;
 	Scorekeeper *sk;
+	GrafPtr oldport;
+	Rect textrect;
+	Str255 pstr;
+	char *str;
 
-	if (draw_instructions_text_proc == NULL)
-	  draw_instructions_text_proc = NewUserItemProc(draw_instructions_text);
-
-	instructionswin = GetNewDialog(dInstructions, nil, (DialogPtr) -1L);
-	GetDItem(instructionswin, diInstructionsTitle, &itemtype, &itemhandle, &itemrect);
-	c2p(((mainmodule && mainmodule->title) ? mainmodule->title : ""), tmpstr);
-	SetIText(itemhandle, tmpstr);
-	GetDItem(instructionswin, diInstructionsText, &itemtype, &itemhandle, &itemrect);
-	SetDItem(instructionswin, diInstructionsText, itemtype, (Handle) draw_instructions_text_proc, &itemrect);
-	destrect = itemrect;
-	viewrect = itemrect;
+	GetPort(&oldport);
 	SetPort(instructionswin);
+
+	/* Draw the game title. */
+	c2p(((mainmodule && mainmodule->title) ? mainmodule->title : ""), pstr);
+	TextFont(title_font_id);
+	TextSize(title_font_size);
+	MoveTo(15, 20);
+	DrawString(pstr);
 	TextFont(small_font_id);
 	TextSize(small_font_size);
-	instructions_text = TENew(&destrect, &viewrect);
+
+	/* Draw a horizontal line under the title. */
+	MoveTo(0, 30);
+	LineTo(instructionswin->portRect.right, 30);
+
+	/* Draw the instructions text itself. */
+	textrect = instructionswin->portRect;
+	textrect.left += 15;
+	textrect.right -= 15;
+	textrect.top += 40;
+	textrect.bottom -= 15;
+	instructions_text = TENew(&textrect, &textrect);
 	TESetSelect(0, 32767, instructions_text);
 	TEDelete(instructions_text);
+
 	/* (should generify, add parameter for how to line break) */
 	if (mainmodule == NULL) {
 		run_error("no main module?");
@@ -230,22 +234,37 @@ create_instructions_dialog()
 			TEInsert(str, strlen(str), instructions_text);
 		}
 	}
-	/* This makes the title item draw big. */
-	TextFont(title_font_id);
-	TextSize(title_font_size);
-	add_window_menu_item("Instructions", instructionswin);
+	SetPort(oldport);
 }
 
-int
-hit_instructions_dialog(DialogPtr dialog, int itemhit, EventRecord *evt)
+/* New function. */
+
+void
+activate_instructions(int activate)
 {
-	switch (itemhit) {
-		case diInstructionsHelp:
-			/* Just jump to the help dialog. */
-			help_dialog(NULL);
-			break;
+	HiliteControl(openhelpbutton, (activate ? 0 : 255));
+	HiliteControl(closewinbutton, (activate ? 0 : 255));
+}
+
+/* New function. Replaces hit_instructions_dialog. */
+
+void
+do_mouse_down_instructions(Point mouse, int mods)
+{
+	ControlHandle control;
+	Handle itemhandle;
+	short part;
+	
+	part = FindControl(mouse, instructionswin, &control);
+	if (control == openhelpbutton) {
+		TrackControl(control, mouse, NULL);
+		close_window(instructionswin);
+	    	show_help_window(NULL);
+	}
+	if (control == closewinbutton) {
+		TrackControl(control, mouse, NULL);
+		close_window(instructionswin);
 	}
-	return TRUE;
 }
 
 /* Help window. */
@@ -254,7 +273,7 @@ hit_instructions_dialog(DialogPtr dialog, int itemhit, EventRecord *evt)
    anywhere, anytime. */
 
 void
-help_dialog(HelpNode *helpnode)
+show_help_window(HelpNode *helpnode)
 {
 	if (helpwin == nil) {
 		create_help_window();
@@ -263,8 +282,8 @@ help_dialog(HelpNode *helpnode)
 		cur_help_node = helpnode;
 		set_help_content(helpnode);
 	}
-	ShowWindow(helpwin);
-	SelectWindow(helpwin);
+	SelectTheWindow(helpwin);
+	ActivateWindow(helpwin, true);
 }
 
 void
@@ -350,31 +369,33 @@ create_help_window()
 
 	/* Create the window, color if possible, since images may be in color. */
 	if (hasColorQD) {	
-		helpwin = GetNewCWindow(wHelp, NULL, (WindowPtr) -1L);
+		helpwin = GetNewCWindow(wHelp, NULL, NULL);
 	} else {
-		helpwin = GetNewWindow(wHelp, NULL, (WindowPtr) -1L);
+		helpwin = GetNewWindow(wHelp, NULL, NULL);
 	}
 	topicsbutton = GetNewControl(cTopicsButton, helpwin);
 	helpbutton = GetNewControl(cHelpButton, helpwin);
 	prevbutton = GetNewControl(cPrevButton, helpwin);
 	nextbutton = GetNewControl(cNextButton, helpwin);
 	backbutton = GetNewControl(cBackButton, helpwin);
+	closebutton = GetNewControl(cCloseButton, helpwin);
+
 	SetPort(helpwin);
 	TextFont(title_font_id);
 	TextSize(title_font_size);
-	SetRect(&helptopicrect, 45, 45, 305, 75); 
+	SetRect(&helptopicrect, 45, 45, 220, 75); 
 	helptopic = TENew(&helptopicrect, &helptopicrect);
 	TextFont(small_font_id);
 	TextSize(small_font_size);
 	h = window_width(helpwin);  v = window_height(helpwin);
-	SetRect(&viewrect, 5, 75, h - sbarwid, v - sbarwid); 
+	SetRect(&viewrect, 5, 75, h - floatsbarwid, v); 
 	destrect = viewrect;
 	helptext = TENew(&destrect, &destrect);
 	/* Set up a vertical scrollbar. */
 	vscrollrect = helpwin->portRect;
-	vscrollrect.top = 75;
-	vscrollrect.bottom -= sbarwid - 1;
-	vscrollrect.left = vscrollrect.right - sbarwid;
+	vscrollrect.top = -1;
+	vscrollrect.bottom -= floatsbarwid - 1;
+	vscrollrect.left = vscrollrect.right - floatsbarwid;
 	vscrollrect.right += 1;
 	helpvscrollbar =
 		NewControl(helpwin, &vscrollrect, "\p", TRUE, 0, 0, 0, scrollBarProc, 0L);
@@ -382,8 +403,8 @@ create_help_window()
 	/* Add the Mac-specific help nodes. */
 	/* (Note that these will appear in *reverse* order from here, because each
 	   is being glued right after the first help node. */
-	add_help_node("textual commands", describe_text_commands, 0, first_help_node);
-	add_help_node("keyboard", describe_keyboard, 0, first_help_node);
+	add_help_node("textual commands", describe_long_commands, 0, first_help_node);
+	add_help_node("keyboard", describe_key_commands, 0, first_help_node);
 	add_help_node("menus", describe_menus, 0, first_help_node);
 	add_help_node("mouse", describe_mouse, 0, first_help_node);
 	help_help_node = add_help_node("help", describe_help, 0, first_help_node);
@@ -394,6 +415,9 @@ create_help_window()
 	  nodestack = (HelpNode **) xmalloc(NODESTACKSIZE * sizeof(HelpNode *));
 	nodestackpos = 0;
 	add_window_menu_item("Help", helpwin);
+
+	/* Finally make it floating. */
+	MakeFloat(helpwin);
 }
 
 void
@@ -409,7 +433,9 @@ set_help_content(HelpNode *curnode)
 	TESetText(curnode->key, strlen(curnode->key), helptopic);
 	/* Set the displayed text. */
 	str = curnode->text;
-#ifdef THINK_C
+
+#if defined(THINK_C) || defined(__MWERKS__)
+
 	/* Hack up newlines so that TextEdit recognizes them. */
 	{
 		int i, len = strlen(str);
@@ -419,7 +445,9 @@ set_help_content(HelpNode *curnode)
 			  str[i] = '\r';
 		}
 	}
+
 #endif
+
 	helpstring = str;
 	/* Remove all the existing text. */
 	TESetSelect(0, 32767, helptext);
@@ -442,7 +470,7 @@ draw_help()
 	SetPort(helpwin);
 	/* Draw a horizontal line under the navigation buttons. */
 	MoveTo(0, 30);
-	LineTo(helpwin->portRect.right - sbarwid, 30);
+	LineTo(helpwin->portRect.right - floatsbarwid, 30);
 	SetRect(&tmprect, 5, 40, 5 + 32, 40 + 32);
 	/* Grow so as to erase extra junk that may have been drawn. */
 	InsetRect(&tmprect, -2, -2);
@@ -452,7 +480,7 @@ draw_help()
 	/* Draw a bit of graphic for some help nodes. */
 	if (cur_help_node->nclass == utypenode && is_unit_type(cur_help_node->arg)) {
 		draw_unit_image(helpwin, tmprect.left, tmprect.top, 32, 32,
-						cur_help_node->arg, -1, 0, FALSE);
+						cur_help_node->arg, 0, 0, 0, 0, FALSE);
 	} else if (cur_help_node->nclass == ttypenode && is_terrain_type(cur_help_node->arg)) {
 		draw_terrain_sample(helpwin, tmprect, cur_help_node->arg); 
 	} else if (cur_help_node->nclass == mtypenode && is_material_type(cur_help_node->arg)) {
@@ -464,6 +492,15 @@ draw_help()
 	TextFont(small_font_id);
 	TextSize(small_font_size);
 	TEUpdate(&(helpwin->portRect), helptext);
+
+	/* Draw a default line around the close button. */
+	DrawControls(helpwin);
+	tmprect = (*closebutton)->contrlRect;
+	PenSize(3, 3);
+	InsetRect(&tmprect, -4, -4);
+	FrameRoundRect(&tmprect, 16, 16);
+	InsetRect(&tmprect,  4,  4);
+	PenNormal();
 	SetPort(oldport);
 	adjust_help_scrollbar();
 }
@@ -487,6 +524,12 @@ void
 activate_help(int activate)
 {
 	HiliteControl(helpvscrollbar, (activate ? 0 : 255));
+	HiliteControl(topicsbutton, (activate ? 0 : 255));
+	HiliteControl(helpbutton, (activate ? 0 : 255));
+	HiliteControl(prevbutton, (activate ? 0 : 255));
+	HiliteControl(nextbutton, (activate ? 0 : 255));
+	HiliteControl(backbutton, (activate ? 0 : 255));
+	HiliteControl(closebutton, (activate ? 0 : 255));
 }
 
 static pascal void
@@ -505,13 +548,13 @@ help_vscroll_fn(ControlHandle control, short code)
 			jump = pagesize;
 			break;
 		case inDownButton:
-			jump = 1;
+			jump = 2;
 			break;
 		case inPageUp:
 			jump = - pagesize;
 			break;
 		case inUpButton:
-			jump = -1;
+			jump = -2;
 			break;
 		default:
 			jump = 0;
@@ -526,6 +569,57 @@ help_vscroll_fn(ControlHandle control, short code)
 	  TEScroll(0, - jump * (*helptext)->lineHeight, helptext);
 }
 
+int 
+do_key_down_help(char key)
+{
+	HelpNode 	*prevhelpnode = cur_help_node;
+	int		i, handled = TRUE;
+	GrafPtr	oldport;
+
+	GetPort(&oldport);
+	SetPort(helpwin);
+
+	switch (key) {
+		case 3:		/* Enter. */
+		case 13:		/* Return. */
+		case 27:		/* Escape. */
+			close_window(helpwin);
+			break;
+		case 0x1F:		/* Down Arrow */
+			help_vscroll_fn(helpvscrollbar, inDownButton);
+			break;
+		case 0x1E:		/* Up Arrow */
+			help_vscroll_fn(helpvscrollbar, inUpButton);
+			break;
+		case 0x1C:		/* Left Arrow */
+			cur_help_node = cur_help_node->prev;
+			break;
+		case 0x1D:		/* Right Arrow */
+			cur_help_node = cur_help_node->next;
+			break;
+		case 0x0C:		/* Page Down */
+			help_vscroll_fn(helpvscrollbar, inPageDown);
+			break;
+		case 0x0B:		/* Page Up */
+			help_vscroll_fn(helpvscrollbar, inPageUp);
+			break;
+		default:
+			handled = FALSE;
+			break;
+	}
+	/* If we changed help nodes, get its contents and record on the node stack. */
+	if (prevhelpnode != cur_help_node) {
+		set_help_content(cur_help_node);
+		if (nodestackpos >= NODESTACKSIZE) {
+			for (i = 1; i < NODESTACKSIZE; ++i) nodestack[i - 1] = nodestack[i];
+			nodestackpos = NODESTACKSIZE - 1;
+		}
+		nodestack[nodestackpos++] = prevhelpnode;
+	}
+	SetPort(oldport);
+	return handled;
+}
+
 /* Respond to an event occurring in the help window. */
 
 void
@@ -541,17 +635,26 @@ do_mouse_down_help(Point mouse, int mods)
 
 	part = FindControl(mouse, helpwin, &control);
 	if (control == topicsbutton) {
+		TrackControl(control, mouse, NULL);
 		cur_help_node = topics_help_node;
 	} else if (control == helpbutton) {
+		TrackControl(control, mouse, NULL);
 		cur_help_node = help_help_node;
 	} else if (control == prevbutton) {
+		TrackControl(control, mouse, NULL);
 		cur_help_node = cur_help_node->prev;
 	} else if (control == nextbutton) {
+		TrackControl(control, mouse, NULL);
 		cur_help_node = cur_help_node->next;
 	} else if (control == backbutton) {
+		TrackControl(control, mouse, NULL);
 		if (nodestackpos > 0) {
 			cur_help_node = nodestack[--nodestackpos];
 		}
+	} else if (control == closebutton) {
+		TrackControl(control, mouse, NULL);
+		close_window(helpwin);
+		
 	} else if (control == helpvscrollbar) {
 		if (part != 0) {
 			switch (part) {
@@ -626,11 +729,11 @@ grow_help(int h, int v)
 {
 	EraseRect(&helpwin->portRect);
 	SizeWindow(helpwin, h, v, 1);
-	MoveControl(helpvscrollbar, h - sbarwid, 75);
-	SizeControl(helpvscrollbar, sbarwid + 1, v - 75 - sbarwid + 1);
-	(*helptext)->viewRect.right = h - sbarwid;
-	(*helptext)->viewRect.bottom = v - sbarwid;
-	(*helptext)->destRect.right = h - sbarwid;
+	MoveControl(helpvscrollbar, h - floatsbarwid, -1);
+	SizeControl(helpvscrollbar, floatsbarwid + 1, v  - floatsbarwid + 2);
+	(*helptext)->viewRect.right = h - floatsbarwid;
+	(*helptext)->viewRect.bottom = v;
+	(*helptext)->destRect.right = h - floatsbarwid;
 	TECalText(helptext);
 	InvalRect(&helpwin->portRect);
 }					
@@ -663,14 +766,14 @@ zoom_help(int part)
 		}
 		(*((WStateDataHandle) ((WindowPeek) helpwin)->dataHandle))->stdState = zoomrect;
 	}
-	ZoomWindow(helpwin, part, (helpwin == FrontWindow()));
+	ZoomWindow(helpwin, part, false);
 	h = window_width(helpwin);  v = window_height(helpwin);
-	MoveControl(helpvscrollbar, h - sbarwid, 0);
-	SizeControl(helpvscrollbar, sbarwid + 1, v - sbarwid + 1);
+	MoveControl(helpvscrollbar, h - floatsbarwid, -1);
+	SizeControl(helpvscrollbar, floatsbarwid + 1, v - floatsbarwid + 2);
 	adjust_help_scrollbar();
-	(*helptext)->viewRect.right = h - sbarwid;
-	(*helptext)->viewRect.bottom = v - sbarwid;
-	(*helptext)->destRect.right = h - sbarwid;
+	(*helptext)->viewRect.right = h - floatsbarwid;
+	(*helptext)->viewRect.bottom = v;
+	(*helptext)->destRect.right = h - floatsbarwid;
 	TECalText(helptext);
 	/* This will force a full redraw at the next update. */
 	InvalRect(&helpwin->portRect);
