@@ -17,7 +17,7 @@ extern char *connection_method_name;
 
 /* Limit on the number of check boxes and sliders in the variants dialog. */
 
-#define MAXCHECKBOXES 6
+#define MAXCHECKBOXES 10
 #define MAXSLIDERS 3
 
 #define toggle_checkbox(ih) \
@@ -942,10 +942,6 @@ variants_dialog()
 	set_variant_item(diVariantsPeople, vary_people, TRUE, new_people);
 	set_variant_item(diVariantsEconomy, vary_economy, TRUE, new_economy);
 	set_variant_item(diVariantsSupply, vary_supply, TRUE, new_supply);
-	set_variant_item(diVariantsIndepsAI, vary_indepside_ai, TRUE, new_indepside_ai);
-	set_variant_item(diVariantsIndepsBuild, vary_indepside_build, TRUE, new_indepside_build);
-	set_variant_item(diVariantsIndepsResearch, vary_indepside_research, TRUE, new_indepside_research);
-	set_variant_item(diVariantsIndepsTreasury, vary_indepside_treasury, TRUE, new_indepside_treasury);
 	set_variant_item(diVariantsWorldSize, vary_world, FALSE, 0);
 	set_variant_item(diVariantsRealTime, vary_real_time, FALSE, 0);
 	set_variant_item(diVariantsMoreVariants, /* more_variants */ FALSE, FALSE, 0);
@@ -984,10 +980,6 @@ variants_dialog()
 			case diVariantsPeople:
 			case diVariantsEconomy:
 			case diVariantsSupply:
-			case diVariantsIndepsAI:
-			case diVariantsIndepsBuild:
-			case diVariantsIndepsResearch:
-			case diVariantsIndepsTreasury:
 				/* Toggle check boxes. */
 				GetDItem(variants_win, ditem, &itemtype, &itemhandle, &itemrect);
 				toggle_checkbox(itemhandle);
