@@ -383,23 +383,25 @@ create_map(int power, int id)
 		win = GetNewWindow((map == worldmap ? wFloatResize : wMap), NULL, NULL);
 	}
 	map->window = win;
+	
+	get_main_screen_size(&mainwidth, &mainheight);
 	if (map == worldmap) {
 	
+		int	winwidth;
+		int	winheight;
 		Point winTL;
-	
+			
 		/* Make worldmapwin just big enough to contain the area. */
-		SizeWindow(win, 
-				area.width * hws[power], 
-				area.height * hcs[power], 
-			FALSE);
+		winwidth = min(area.width * hws[power], mainwidth / 4);
+		winheight = min(area.height * hcs[power], mainheight / 4);
+		SizeWindow(win, winwidth, winheight,	FALSE);
 		/* Put it in the right corner of the frontmap if it exists. */
 		if (frontmap) {
-			winTL.h = frontmap->window->portRect.right - area.width * hws[power] + 7;
+			winTL.h = frontmap->window->portRect.right - winwidth + 7;
 			winTL.v = GetMBarHeight() + 15 + 21;
 		/* Else put it in the right corner of the main screen. */
 		} else {					
-			get_main_screen_size(&mainwidth, &mainheight);
-			winTL.h = mainwidth - area.width * hws[power] - 3;
+			winTL.h = mainwidth - winwidth - 3;
 			winTL.v = GetMBarHeight() + 15;
 		}
 		/* Move it out of the way of game window if necessary. */
@@ -410,7 +412,6 @@ create_map(int power, int id)
 	} else {
 		stagger_window(win, &lastmaph, &lastmapv);
 		if (first_windows) {
-			get_main_screen_size(&mainwidth, &mainheight);
 			/* Use the main screen size if full-sized map desired, else
 			   leave room for game window at side and notices at bottom. */
 			SizeWindow(win,
