@@ -1,5 +1,5 @@
 /* Menus and commands for the Mac interface to Xconq.
-   Copyright (C) 1992-1998 Stanley T. Shebs.
+   Copyright (C) 1992-1998, 2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -524,7 +524,7 @@ do_menu_command(long which)
 	DGprintf("menu %d, item %d\n", menuid, menuitem);
 
 	/* Access world map if control key is down. */	
-	GetKeys((unsigned long *) keyMap);
+	GetKeys((GETKEYS_ARG_TYPE) keyMap);
 	optionKey = keyMap[0x3A >> 3] >> (0x3A & 7) & 1;
 	if (optionKey && worldmap)
 		map = worldmap;
@@ -1844,7 +1844,7 @@ adjust_menus()
 	WindowPtr win;
 
 	/* Access world map if option key is down. */	
-	GetKeys((unsigned long *) keyMap);
+	GetKeys((GETKEYS_ARG_TYPE) keyMap);
 	optionKey = keyMap[0x3A >> 3] >> (0x3A & 7) & 1;
 	if (optionKey && worldmap) {
 		win = worldmapwin;
