@@ -1,5 +1,5 @@
 /* Lower-level drawing routines for the X11 interface to Xconq.
-   Copyright (C) 1987-1989, 1991-1998 Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@ any later version.  See the file COPYING.  */
    the unit info, the map view itself, and the list of sides. */
 
 #include "conq.h"
-#include "xconq.h"
+#include "xtconq.h"
 
 int tmpdrawlighting;
 int tmpdrawcoverage;
@@ -25,69 +25,60 @@ Map *tmpmap;
 
 #define conns_to_draw(m) (numconntypes > 0 && cwid[(m)->vp->power] > 0)
 
-Unit *x_find_unit_or_occ PARAMS ((Side *side, Map *map, Unit *unit,
-				 int usx, int usy, int usw, int ush,
-				 int sx, int sy));
-Unit *x_find_unit_at PARAMS ((Side *side, Map *map, int x, int y,
-			     int sx, int sy));
+Unit *x_find_unit_or_occ(Side *side, Map *map, Unit *unit,
+			 int usx, int usy, int usw, int ush, int sx, int sy);
+Unit *x_find_unit_at(Side *side, Map *map, int x, int y, int sx, int sy);
 
-static void xform_fractional PARAMS ((Side *side, Map *map,
-				      int x, int y, int xf, int yf,
-				      int *sxp, int *syp));
+static void xform_fractional(Side *side, Map *map,
+			     int x, int y, int xf, int yf, int *sxp, int *syp);
 
-static void draw_feature_name PARAMS ((Side *side, Map *map, int fid));
-static void draw_border_line_mult PARAMS ((Side *side, Map *map,
-					   Window win, int sx, int sy,
-					   int bitmask, int power, int t));
-static void draw_connection_line_mult PARAMS ((Side *side, Window win,
-					       int sx, int sy, int bitmask,
-					       int power, int c));
-static void draw_hex_polygon PARAMS ((Side *side, Map *map, Window win,
-				      GC gc, int sx, int sy,
-				      int power, int over, int dogrid));
-static void draw_area_background PARAMS ((Side *side, Map *map));
-static void meridian_line_callback PARAMS ((int x1, int y1, int x1f, int y1f,
-					    int x2, int y2, int x2f, int y2f));
-static void meridian_text_callback PARAMS ((int x1, int y1, int x1f, int y1f,
-					    char *str));
-static int cell_drawing_info PARAMS ((Side *side, int x, int y, int power,
-				      int seeall, Pixmap *patp, long *colorp));
-static void set_terrain_gc_for_image PARAMS ((Side *side, Image *timg));
-static void draw_terrain_row PARAMS ((Side *side, Map *map,
-				      int x0, int y0, int len, int force));
-static void draw_contours PARAMS ((Side *side, Map *map,
-				   int x0, int y0, int len));
-static void draw_elevations PARAMS ((Side *side, Map *map,
-				     int x0, int y0, int len));
-static void draw_clouds_row PARAMS ((Side *side, Map *map,
-				     int x0, int y0, int len));
-static void draw_temperature_row PARAMS ((Side *side, Map *map,
-					  int x0, int y0, int len));
-static void draw_winds_row PARAMS ((Side *side, Map *map,
-				    int x0, int y0, int len));
-static void draw_units PARAMS ((Side *side, Map *map, int x, int y));
-static void draw_unit_and_occs PARAMS ((Side *side, Map *map, Unit *unit,
-					int sx, int sy, int sw, int sh,
-					int drawoccs));
-static void draw_unit_name PARAMS ((Side *side, Map *map, Unit *unit,
-				   int sx, int sy, int sw, int sh));
-static void draw_people PARAMS ((Side *side, Map *map, int x, int y));
-static void draw_borders PARAMS ((Side *side, Map *map, int x, int y, int b));
-static void draw_connections PARAMS ((Side *side, Map *map, int x, int y,
-				      int c));
-static void draw_legend PARAMS ((Side *side, Map *map, int x, int y));
-static void draw_country_border_line PARAMS ((Side *side, Window win,
-					      int sx, int sy, int dir,
-					      int power));
-static void draw_legend_text PARAMS ((Side *side, Window win,
-				      int sx, int sy, int power, char *str,
-				      int color, int maskit));
-static void draw_feature_boundary PARAMS ((Side *side, Map *map,
-					   int x, int y, int fid));
-static void draw_info_text PARAMS ((Side *side, Map *map, int x, int y,
-				    int len, char *buf));
+static void draw_feature_name(Side *side, Map *map, int fid);
+static void draw_border_line_mult(Side *side, Map *map, Window win,
+				  int sx, int sy, int bitmask, int power,
+				  int t);
+static void draw_connection_line_mult(Side *side, Window win,
+				      int sx, int sy, int bitmask,
+				      int power, int c);
+static void draw_hex_polygon(Side *side, Map *map, Window win,
+			     GC gc, int sx, int sy,
+			     int power, int over, int dogrid);
+static void draw_area_background(Side *side, Map *map);
+static void meridian_line_callback(int x1, int y1, int x1f, int y1f,
+				   int x2, int y2, int x2f, int y2f);
+static void meridian_text_callback(int x1, int y1, int x1f, int y1f,
+				   char *str);
+static int cell_drawing_info(Side *side, int x, int y, int power,
+			     int seeall, Pixmap *patp, long *colorp);
+static void set_terrain_gc_for_image(Side *side, Image *timg);
+static void draw_terrain_row(Side *side, Map *map,
+			     int x0, int y0, int len, int force);
+static void draw_contours(Side *side, Map *map, int x0, int y0, int len);
+#if 0
+static void draw_elevations(Side *side, Map *map, int x0, int y0, int len);
+#endif
+static void draw_clouds_row(Side *side, Map *map, int x0, int y0, int len);
+static void draw_temperature_row(Side *side, Map *map, int x0, int y0,
+				 int len);
+static void draw_winds_row(Side *side, Map *map, int x0, int y0, int len);
+static void draw_units(Side *side, Map *map, int x, int y);
+static void draw_unit_and_occs(Side *side, Map *map, Unit *unit,
+			       int sx, int sy, int sw, int sh, int drawoccs);
+static void draw_unit_name(Side *side, Map *map, Unit *unit,
+			   int sx, int sy, int sw, int sh);
+static void draw_people(Side *side, Map *map, int x, int y);
+static void draw_borders(Side *side, Map *map, int x, int y, int b);
+static void draw_connections(Side *side, Map *map, int x, int y, int c);
+static void draw_legend(Side *side, Map *map, int x, int y);
+static void draw_country_border_line(Side *side, Window win,
+				     int sx, int sy, int dir, int power);
+static void draw_legend_text(Side *side, Window win,
+			     int sx, int sy, int power, char *str,
+			     int color, int maskit);
+static void draw_feature_boundary(Side *side, Map *map, int x, int y, int fid);
+static void draw_info_text(Side *side, Map *map, int x, int y,
+			   int len, char *buf);
 
-static void draw_meridians PARAMS ((Side *side, Map *map));
+static void draw_meridians(Side *side, Map *map);
 
 #define cell_overlay(side, x, y)  \
   ((side->designer) \
@@ -474,7 +465,7 @@ Map *map;
 	/* (should test for input events here, would respond better) */
     }
     /* Now draw the lat-long grid if asked to do so. */
-    if (map->vp->draw_latlong && map->vp->latlong_interval > 0)
+    if (map->vp->draw_meridians && map->vp->meridian_interval > 0)
       draw_meridians(side, map);
     draw_current(side, map);
 }
@@ -489,8 +480,8 @@ Map *map;
 
     tmpside = side;
     tmpmap = map;
-    XSetForeground(dpy, gc, side->ui->blackcolor);
-    XSetBackground(dpy, gc, side->ui->whitecolor);
+    XSetForeground(dpy, gc, side->ui->meridian_color);
+    XSetBackground(dpy, gc, side->ui->bgcolor);
     XSetClipMask(dpy, gc, None);
     XSetLineAttributes(dpy, gc, 1, LineSolid, CapButt, JoinMiter); 
     plot_meridians(map->vp, meridian_line_callback, meridian_text_callback);
@@ -513,6 +504,11 @@ meridian_text_callback(x1, y1, x1f, y1f, str)
 int x1, y1, x1f, y1f;
 char *str;
 {
+    int sx1, sy1;
+
+    xform_fractional(tmpside, tmpmap, x1, y1, x1f, y1f, &sx1, &sy1);
+    draw_text(tmpside, tmpmap->viewwin, sx1 + 1, sy1 + 1, str,
+	      tmpside->ui->fgcolor);
 }
 
 /* The basic map drawing routine does an entire row at a time, which yields
@@ -579,13 +575,13 @@ int x0, y0, len, clearit;
     }
     if (len <= 0)
       return;
-    if (features_defined() && map->drawfeatureboundaries) {
+    if (features_defined() && map->vp->draw_feature_boundaries) {
 	for (x = x0; x < x0 + len; ++x) {
 	    draw_feature_boundary(side, map, x, y0, raw_feature_at(x, y0));
 	}
     }
-    if (features_defined() && map->drawfeaturenames && side->ui->legends) {
-	for (i = 0; i < side->ui->numfeatures; ++i) {
+    if (features_defined() && map->vp->draw_feature_names && side->ui->legends) {
+	for (i = 0; i < numfeatures; ++i) {
 	    if (side->ui->legends[i].oy == y0) {
 		draw_feature_name(side, map, i);
 	    }
@@ -638,8 +634,7 @@ int f;
       return;
     
     XSetFillStyle(side->ui->dpy, side->ui->gc, FillSolid);
-    /* (should choose a color contrasting with main terrain) */
-    XSetForeground(side->ui->dpy, side->ui->gc, side->ui->fgcolor);
+    XSetForeground(side->ui->dpy, side->ui->gc, side->ui->feature_color);
     xform(side, map, x, y, &sx0, &sy0);
     /* xform returns coordinates of the upper-left corner of the cell */
     sxc2 = 2 * sx0 + (leg->dx + 1) * map->vp->hw; /* twice center x */
@@ -903,13 +898,10 @@ int x0, y0, len;
     Display *dpy = side->ui->dpy;
     Window win = map->viewwin;
     GC gc = side->ui->gc;
-    extern int contour_interval;
 
-    init_contour_lines();
-    if (contour_interval < 1)
+    if (map->vp->contour_interval < 1)
       return;
-    XSetForeground(dpy, gc, side->ui->blackcolor);
-    XSetBackground(dpy, gc, side->ui->whitecolor);
+    XSetForeground(dpy, gc, side->ui->contour_color);
     XSetClipMask(dpy, gc, None);
     XSetLineAttributes(dpy, gc, 1, LineSolid, CapButt, JoinMiter); 
     y = y0;
@@ -926,6 +918,7 @@ int x0, y0, len;
     }
 }
 
+#if 0
 static void
 draw_elevations(side, map, x0, y0, len)
 Side *side;
@@ -946,6 +939,7 @@ int x0, y0, len;
 	}
     }
 }
+#endif
 
 static void
 draw_clouds_row(side, map, x0, y0, len)
@@ -1032,7 +1026,7 @@ int x, y;
 	    }
 	    if (unit == NULL) {
 		for_all_stack(x, y, unit) {
-		    if (map->seeall || unit_actually_visible(side, unit))
+		    if (unit_visible(side, map->vp, unit))
 		      break;
 		}
 	    }
@@ -1081,7 +1075,7 @@ int x, y;
 	} else {
 	    didone = FALSE;
 	    for_all_stack(x, y, unit) {
-		if (unit_actually_visible(side, unit)) {
+		if (unit_visible(side, map->vp, unit)) {
 		    x_xform_unit(side, map, unit, &sx, &sy, &sw, &sh);
 		    draw_unit_and_occs(side, map, unit, sx, sy, sw, sh, TRUE);
 		    didone = TRUE;
@@ -1101,28 +1095,6 @@ int x, y;
     }
 }
 
-/* (should be generic?) */
-int
-unit_actually_visible(side, unit)
-Side *side;
-Unit *unit;
-{
-    int uview;
-
-    if (unit->side == side)
-      return TRUE;
-    if (side->see_all)
-      return TRUE;
-    if (u_see_always(unit->type))
-      return TRUE;
-    /* (should be true if see chance >= 100) */
-    uview = unit_view(side, unit->x, unit->y);
-    if (unit->type == vtype(uview)
-	&& side_number(unit->side) == vside(uview))
-      return TRUE;
-    return FALSE;
-}
-
 static void
 draw_unit_and_occs(side, map, unit, sx, sy, sw, sh, drawoccs)
 Side *side;
@@ -1396,7 +1368,7 @@ Map *map;
        that used all acp, gray if the unit can't do anything. */
     shade = (unit ?
 	     ((unit->act && unit->act->initacp > 0) ?
-	      ((unit->act->acp > 0) ? black : darkgray) :
+	      (has_acp_left(unit) ? black : darkgray) :
 	      gray) :
 	     black);
     /* Box at larger sizes need to come in slightly, otherwise we
@@ -1527,9 +1499,9 @@ int x, y, fid;
 
     /* for now: */
     if (fid0 == side->ui->curfid) {
-	color = side->ui->enemycolor;
+	color = side->ui->badcolor;
     } else {
-	color = side->ui->neutcolor;
+	color = side->ui->graycolor;
     }
     XSetForeground(dpy, gc, color);
     if (side->ui->monochrome) {
@@ -1701,7 +1673,7 @@ Side *side;
 Window win;
 int sx, sy, sw, sh, u, s2, fg, bg, mod;
 {
-    char buf[1], *ename;
+    char *ename;
     int sx2, sy2, ex, ey, ew, eh, desperate = FALSE;
     long imagecolor = side->ui->blackcolor, maskcolor = side->ui->whitecolor;
     Image *uimg;
@@ -1715,6 +1687,7 @@ int sx, sy, sw, sh, u, s2, fg, bg, mod;
     /* Just draw a small box at 4x4 and below. */
     if (sw <= 4) {
 	XSetClipMask(dpy, gc, None);
+	/* (should draw with a side color if possible) */
 	XSetForeground(dpy, gc, side->ui->blackcolor);
 	XFillRectangle(dpy, win, gc, sx, sy, sw, sh);
 	return;
@@ -1793,8 +1766,6 @@ int sx, sy, sw, sh, u, s2, fg, bg, mod;
 	/* For some reason, we have no useful image; draw a blank box
 	   instead. */
 	XSetClipOrigin(dpy, gc, sx, sy);
-	/* (should actually clip to cell boundaries, maybe even inside
-	   border terrain if necessary?) */
 	XSetClipMask(dpy, gc, None);
 	/* Draw an edge, at least for larger mags. */
 	if (sw >= 16) {
@@ -1832,7 +1803,7 @@ Side *side;
 Window win;
 int ex, ey, ew, eh, s2, style;
 {
-    int ex2, ey2, ew2, eh2, offset;
+    int ex2, ey2, offset;
     long imagecolor, maskcolor;
     Image *eimg;
     X11Image *ximg;
@@ -1914,7 +1885,7 @@ int sx, sy, dir, power;
     if (wid == 0)
       return;
     wid = max(1, wid / 2);
-    XSetForeground(dpy, gc, side->ui->whitecolor);
+    XSetForeground(dpy, gc, side->ui->country_border_color);
     XSetClipMask(dpy, gc, None);
     XSetLineAttributes(dpy, gc, wid, LineSolid, CapButt, JoinMiter); 
     XDrawLine(dpy, win, gc,
@@ -1930,21 +1901,19 @@ Side *side;
 Map *map;
 {
     char infobuf[BUFSIZE];
-    int u, s, t, mrow, x = map->curx, y = map->cury, uview, len;
+    int u, s, mrow, x = map->curx, y = map->cury, uview, len;
     Unit *unit, *unit2;
     Side *side2 = NULL; /* init to keep GCC quiet */
 
     len = 40;
     XClearWindow(side->ui->dpy, map->infowin);
-    /* (should make generic) */
     unit = map->curunit;
     if (!in_play(unit)) {
-	/* (should pick a "most visible" unit) */
-	unit2 = unit_at(x, y);
 	if (inside_area(x, y)) {
-	    if ((map->seeall || units_visible(side, x, y))
-		&& unit2 != NULL
-		&& (map->seeall || unit_actually_visible(side, unit2))) {
+	    /* (should pick the "most visible" unit at x,y) */
+	    unit2 = unit_at(x, y);
+	    if (unit2 != NULL
+		&& unit_visible(side, map->vp, unit2)) {
 		sprintf(infobuf, "%s", unit_handle(side, unit2));
 		draw_info_text(side, map, 0, 0, len, infobuf);
 		location_desc(infobuf, side, unit2, unit2->type, x, y);
@@ -1964,8 +1933,6 @@ Map *map;
 		location_desc(infobuf, side, NULL, u, x, y);
 		draw_info_text(side, map, 0, 1, len, infobuf);
 	    }
-	} else {
-	    /* (should never happen?) */
 	}
 	return;
     }
@@ -1984,7 +1951,15 @@ Map *map;
     strcat(infobuf, "   ");
     acp_desc(tmpbuf, unit, TRUE);
     strcat(infobuf, tmpbuf);
-    /* (should display cxp and morale here also, if defined) */
+    cxp_desc(tmpbuf, unit, TRUE);
+    strcat(infobuf, tmpbuf);
+    morale_desc(tmpbuf, unit, TRUE);
+    strcat(infobuf, tmpbuf);
+    /* Crude hack, should be replaced with something better */
+    if (supply_system_in_use()) 
+      sprintf(&infobuf[strlen(infobuf)],
+	      "   Supply inflow=%d%% connectedness=%d%%",
+	      supply_inflow(unit), supply_connectedness(unit));
     draw_info_text(side, map, 50, 0, -1, infobuf);
     /* List other stack members here. */
     others_here_desc(infobuf, unit);
@@ -2004,7 +1979,7 @@ Map *map;
 	plan_desc(infobuf, unit);
 	draw_info_text(side, map, 0, 3, len, infobuf);
 	for (task = unit->plan->tasks; task != NULL; task = task->next) {
-	    task_desc(infobuf, unit->side, task);
+	    task_desc(infobuf, unit->side, unit, task);
 	    draw_info_text(side, map, 0, row++, len, infobuf);
 	}
     }
@@ -2071,7 +2046,6 @@ Map *map;
     sy = (side_number(side2) - 1) * map->sidespacing;
     draw_side_emblem(side, win, 1, sy + 2, 16, 16, side_number(side2), 1);
     /* Build up and write the textual description of the side. */
-    /* (should make generic?) */
     sidebuf[0] = '\0';
 #ifdef DESIGNERS
     if (side2->designer)
@@ -2087,7 +2061,7 @@ Map *map;
 	strcat(sidebuf, ")");
     }
     draw_text(side, win, sx, sy, sidebuf,
-    	      (side2 == side ? side->ui->bgcolor : side->ui->fgcolor));
+	      (side2 == side ? side->ui->bgcolor : side->ui->fgcolor));
     if (keeping_score() && side2->everingame) {
 	Scorekeeper *sk;
 	char *scoredesc;
